#include <stdlib.h>
#include <iostream.h>
#include <string.h>

#include "TROOT.h"
#include "TFile.h"
 
#include "TH2.h"
#include "TH1.h"
#include "TCanvas.h"
#include "TPad.h"

//______________________________________________________________________________

#include "TPhTestData.h"

#include "TPhSetup.h"
#include "TPhEvent.h"

#include "TPhDetFamily.h"
#include "TPhWA98Fam.h"
#include "TPhTeleFamily.h"
#include "TPhPhobFamily.h"

#include "TPhDetector.h"
#include "TPhTeleDetector.h"
#include "TPhPhobDetector.h"

#include "TPhDetHit.h"
#include "TPhPad.h"
#include "TPhPed.h"
#include "TPhWA98Pad.h"

const Int_t TPhPhobFamily::kFamID(5);
const Int_t TPhTeleFamily::kFamID(7);
const Int_t TPhTeleFamily::kChanVA1(128);
const Int_t TPhWA98Family::kFamID(12);
const Int_t TPhWA98Family::kDataSize(4096);
const Int_t TPhWA98Family::kNumRadius(22);
const Int_t TPhTestData::kKEY = Int_t(1997);
const Int_t TPhTestData::kHEADSIZE = Int_t(80);

main(int argc, char **argv)
{
	TROOT simple("simple","TestBeam Analysis");

	char * run = "170";
	Int_t  runI = 170;
	char * indrive = "D";
	char * outdrive = "D";
	Int_t nevent = -1;

   if (argc > 1)  run      = argv[1];
   if (argc > 1)  runI     = atoi(argv[1]);
   if (argc > 2)  indrive  = argv[2];
   if (argc > 3)  outdrive = argv[3];
   if (argc > 4)  nevent   = atoi(argv[4]);

	cout << "Input is " << endl;
	cout << "Run <" << run << ">" << endl;
	cout << "Input Drive <" << indrive << ">" << endl;
	cout << "Output Drive <" << outdrive << ">" << endl;
	if (nevent > 0)
	{
		cout << "Nevent = " << nevent << endl;
	}
	else
	{
		cout << "Read entire File" << endl;
	}

	TFile *hfile;

	TPhEvent * event = new TPhEvent();

// Setup Phob
	TPhPhobFamily * phob = new TPhPhobFamily();
	event->AddFamily(phob);
	phob->ReadGeometry("Phobgeom.dat","phobGeometry.dat");

// Setup Tele
	TPhTeleFamily * tele = new TPhTeleFamily();
	event->AddFamily(tele);
	tele->ReadGeometry("Telegeom.dat");

// Setup WA98
	TPhWA98Family * wa98 = new TPhWA98Family();
	event->AddFamily(wa98);
	wa98->ReadGeometry("WA98geom.dat","WA98.txt");

// Setup All Detectors
	Int_t i;

    Int_t nphob = phob->GetNDetectors();
	TPhDetector ** dphob;
	dphob = new TPhDetector * [nphob];
	Int_t * sphob;
	sphob = new Int_t[nphob];
	for (i = 0; i < nphob; i++)
	{
		dphob[i] = phob->GetDetector(i+1);
	    sphob[i] = dphob[i]->GetNChannels();
	}

	Int_t ntele = tele->GetNDetectors();
	TPhDetector ** dtele;
	dtele = new TPhDetector * [ntele];
	Int_t * stele;
	stele = new Int_t[ntele];
	for (i = 0; i < ntele; i++)
	{
		dtele[i] = tele->GetDetector(i+1);
	    stele[i] = dtele[i]->GetNChannels();
	}

    Int_t nwa98 = wa98->GetNDetectors();
	TPhDetector ** dwa98;
	dwa98 = new TPhDetector *[nwa98];
	Int_t * swa98;
	swa98 = new Int_t[nwa98];
	for (i = 0; i < nwa98; i++)
	{
		dwa98[i] = wa98->GetDetector(i+1);
	    swa98[i] = dwa98[i]->GetNChannels();
	}

	
// arbitray cuts for now

	Int_t   ctele[6] = {20,20,20,20,20,50};
	Int_t   cphob[1] = {75};
	Int_t   cwa98[4] = {15,15,15,15};

// Check Detectctor Setup and add cuts:

	cout << endl;

	cout << "Phobos has " << nphob << " Detectors" << endl;
	for (i = 0; i < nphob; i++)
	{
		dphob[i]->SetCut(cphob[i]);
		cout << "Detector #" << (i+1) << " has " << sphob[i] << " channels.  From " << dphob[i]->GetHitIDMin() << " to " << dphob[i]->GetHitIDMax() << endl;
	}
	cout << endl; 

	cout << "Telescope has " << ntele << " Detectors" << endl;
	for (i = 0; i < ntele; i++)
	{
		dtele[i]->SetCut(ctele[i]);
		cout << "Detector #" << (i+1) << " has " << stele[i] << " channels.  From " << dtele[i]->GetHitIDMin() << " to " << dtele[i]->GetHitIDMax() << endl;
	}
	cout << endl; 

	cout << "WA98 has " << nwa98 << " Detectors" << endl;
	for (i = 0; i < nwa98; i++)
	{
		dwa98[i]->SetCut(cwa98[i]);
		cout << "Detector #" << (i+1) << " has " << swa98[i] << " channels.  From " << dwa98[i]->GetHitIDMin() << " to " << dwa98[i]->GetHitIDMax() << endl;
	}

	cout << endl; 
	
// Create File Names from run and drive
	
	char runfile[81];
	char pedwa98[81];
	char pedtele[81];
	char pedphob[81];
	char rootfile[81];

	char * nmwa98 = "WA98";
    char * nmtele = "Tele";
    char * nmphob = "Phob";

	strcpy(pedwa98, "");
	strncat(pedwa98, outdrive, 1);
	strcat(pedwa98, ":\\Ped_");
	strcat(pedwa98, nmwa98);
	strcat(pedwa98, "_Run");
	strncat(pedwa98, "00000", 5-strlen(run));
	strcat(pedwa98, run);
	strcat(pedwa98, ".dat");

	strcpy(pedphob, "");
	strncat(pedphob, outdrive, 1);
	strcat(pedphob, ":\\Ped_");
	strcat(pedphob, nmphob);
	strcat(pedphob, "_Run");
	strncat(pedphob, "00000", 5-strlen(run));
	strcat(pedphob, run);
	strcat(pedphob, ".dat");

	strcpy(pedtele, "");
	strncat(pedtele, outdrive, 1);
	strcat(pedtele, ":\\Ped_");
	strcat(pedtele, nmtele);
	strcat(pedtele, "_Run");
	strncat(pedtele, "00000", 5-strlen(run));
	strcat(pedtele, run);
	strcat(pedtele, ".dat");

	strcpy(runfile, "");
	strncat(runfile, indrive, 1);
	strcat(runfile, ":\\RUN");
	strncat(runfile, "00000", 5-strlen(run));
	strcat(runfile, run);
	strcat(runfile, ".dat");

	strcpy(rootfile, "");
	strncat(rootfile, outdrive, 1);
	strcat(rootfile,":\\RootFile_Run");
        strncat(rootfile, "00000", 5-strlen(run));
	strcat(rootfile, run);
	strcat(rootfile, ".root");
	

	cout << "Pedfile1 is: " << pedphob << endl;
        cout << "Pedfile2 is: " << pedtele << endl;
	cout << "Pedfile3 is: " << pedwa98 << endl;
	cout << "RunFile  is: " << runfile << endl;
	cout << "RootFile is: " << rootfile << endl;

// Open Root File

	hfile = new TFile(rootfile,"RECREATE","Testbeam analysis");

// Make histograms 


	Int_t x;
	Int_t nchanp;
	Int_t xhighp;
	Int_t xlowp;
	Int_t nchand;
	Int_t xhighd;
	Int_t xlowd;
	Int_t xhighm;
	Int_t xlowm;
	Int_t nchanm;

	Float_t ped;
	Int_t   raw;
	Float_t sub;

	Int_t   nhit;

	cout <<  "Booking Histograms..." << endl;
// set histogram sizes

	xhighp = 250;
	nchanp = 250;
	xlowp  = 0;

	xhighd = 200;
	xlowd  = -50;
	nchand = 250;

	xhighm = 100;
	xlowm  =   0;
    nchanm = 100;

// pedestal and pedestal subtracted

	TH2S * t1p = new TH2S("t1p","Telescope 1 pedestal",stele[0],.5,stele[0]+.5,nchanp,xlowp,xhighp);

	TH2S * t2p = new TH2S("t2p","Telescope 2 pedestal",stele[1],.5,stele[1]+.5,nchanp,xlowp,xhighp);
	TH2S * t3p = new TH2S("t3p","Telescope 3 pedestal",stele[2],.5,stele[2]+.5,nchanp,xlowp,xhighp);
	TH2S * t4p = new TH2S("t4p","Telescope 4 pedestal",stele[3],.5,stele[3]+.5,nchanp,xlowp,xhighp);
	TH2S * t5p = new TH2S("t5p","Telescope 5 pedestal",stele[4],.5,stele[4]+.5,nchanp,xlowp,xhighp);
	TH2S * t6p = new TH2S("t6p","Telescope 6 pedestal",stele[5],.5,stele[5]+.5,nchanp,xlowp,xhighp);

	TH2S * tp[6] = {t1p,t2p,t3p,t4p,t5p,t6p};

	TH2S * t1pp = new TH2S("t1pp","Telescope 1 pedestal",stele[0],.5,stele[0]+.5,nchanp,xlowp,xhighp);
	TH2S * t2pp = new TH2S("t2pp","Telescope 2 pedestal",stele[1],.5,stele[1]+.5,nchanp,xlowp,xhighp);
	TH2S * t3pp = new TH2S("t3pp","Telescope 3 pedestal",stele[2],.5,stele[2]+.5,nchanp,xlowp,xhighp);
	TH2S * t4pp = new TH2S("t4pp","Telescope 4 pedestal",stele[3],.5,stele[3]+.5,nchanp,xlowp,xhighp);
	TH2S * t5pp = new TH2S("t5pp","Telescope 5 pedestal",stele[4],.5,stele[4]+.5,nchanp,xlowp,xhighp);
	TH2S * t6pp = new TH2S("t6pp","Telescope 6 pedestal",stele[5],.5,stele[5]+.5,nchanp,xlowp,xhighp);

	TH2S * tpp[6] = {t1pp,t2pp,t3pp,t4pp,t5pp,t6pp};

	TH2S * t1d = new TH2S("t1d","Telescope 1 signal  ",stele[0],.5,stele[0]+.5,nchand,xlowd,xhighd);
	TH2S * t2d = new TH2S("t2d","Telescope 2 signal  ",stele[1],.5,stele[1]+.5,nchand,xlowd,xhighd);
	TH2S * t3d = new TH2S("t3d","Telescope 3 signal  ",stele[2],.5,stele[2]+.5,nchand,xlowd,xhighd);
	TH2S * t4d = new TH2S("t4d","Telescope 4 signal  ",stele[3],.5,stele[3]+.5,nchand,xlowd,xhighd);
	TH2S * t5d = new TH2S("t5d","Telescope 5 signal  ",stele[4],.5,stele[4]+.5,nchand,xlowd,xhighd);
	TH2S * t6d = new TH2S("t6d","Telescope 6 signal  ",stele[5],.5,stele[5]+.5,nchand,xlowd,xhighd);

	TH2S * td[6] = {t1d,t2d,t3d,t4d,t5d,t6d};

	TH1F * t1d1 = new TH1F("t1d1","Telescope signal",nchand,xlowd,xhighd);
	TH1F * t2d1 = new TH1F("t2d1","Telescope signal",nchand,xlowd,xhighd);
	TH1F * t3d1 = new TH1F("t3d1","Telescope signal",nchand,xlowd,xhighd);
	TH1F * t4d1 = new TH1F("t4d1","Telescope signal",nchand,xlowd,xhighd);
	TH1F * t5d1 = new TH1F("t5d1","Telescope signal",nchand,xlowd,xhighd);
	TH1F * t6d1 = new TH1F("t6d1","Telescope signal",nchand,xlowd,xhighd);

	TH1F * td1[6] = {t1d1,t2d1,t3d1,t4d1,t5d1,t6d1};

	TH2S * p1p = new TH2S("p1p","Phobos 1 pedestal",sphob[0],.5,sphob[0]+.5,nchanp,xlowp,xhighp);

	TH2S * pp[1] = {p1p};
	
	TH2S * p1pp = new TH2S("p1pp","Phobos 1 pedestal",sphob[0],.5,sphob[0]+.5,nchanp,xlowp,xhighp);

	TH2S * ppp[1] = {p1pp};
	
	TH2S * p1d = new TH2S("p1d","Phobos 1 signal  ",sphob[0],.5,sphob[0]+.5,nchand,xlowd,xhighd);

	TH2S * pd[1] = {p1d};

	TH1F * p1d1 = new TH1F("p1d1","Phobos signal",nchand,xlowd,xhighd+100);

	TH1F * pd1[1] = {p1d1};

	TH2S * w1p = new TH2S("w1p","WA98 1 pedestal",swa98[0],.5,swa98[0]+.5,nchanp,xlowp,xhighp);
	TH2S * w2p = new TH2S("w2p","WA98 2 pedestal",swa98[1],.5,swa98[1]+.5,nchanp,xlowp,xhighp);
	TH2S * w3p = new TH2S("w3p","WA98 3 pedestal",swa98[2],.5,swa98[2]+.5,nchanp,xlowp,xhighp);
	TH2S * w4p = new TH2S("w4p","WA98 4 pedestal",swa98[3],.5,swa98[3]+.5,nchanp,xlowp,xhighp);

	TH2S * wp[4] = {w1p,w2p,w3p,w4p};
	
	TH2S * w1pp = new TH2S("w1pp","WA98 1 pedestal",swa98[0],.5,swa98[0]+.5,nchanp,xlowp,xhighp);
	TH2S * w2pp = new TH2S("w2pp","WA98 2 pedestal",swa98[1],.5,swa98[1]+.5,nchanp,xlowp,xhighp);
	TH2S * w3pp = new TH2S("w3pp","WA98 3 pedestal",swa98[2],.5,swa98[2]+.5,nchanp,xlowp,xhighp);
	TH2S * w4pp = new TH2S("w4pp","WA98 4 pedestal",swa98[3],.5,swa98[3]+.5,nchanp,xlowp,xhighp);

	TH2S * wpp[4] = {w1pp,w2pp,w3pp,w4pp};
	
	TH2S * w1d = new TH2S("w1d","WA98 1 signal  ",swa98[0],.5,swa98[0]+.5,nchand,xlowd,xhighd);
	TH2S * w2d = new TH2S("w2d","WA98 2 signal  ",swa98[1],.5,swa98[1]+.5,nchand,xlowd,xhighd);
	TH2S * w3d = new TH2S("w3d","WA98 3 signal  ",swa98[2],.5,swa98[2]+.5,nchand,xlowd,xhighd);
	TH2S * w4d = new TH2S("w4d","WA98 4 signal  ",swa98[3],.5,swa98[3]+.5,nchand,xlowd,xhighd);

	TH2S * wd[4] = {w1d,w2d,w3d,w4d};

	TH1F * w1d1 = new TH1F("w1d1","WA98 signal",nchand,xlowd,xhighd);
	TH1F * w2d1 = new TH1F("w2d1","WA98 signal",nchand,xlowd,xhighd);
	TH1F * w3d1 = new TH1F("w3d1","WA98 signal",nchand,xlowd,xhighd);
	TH1F * w4d1 = new TH1F("w4d1","WA98 signal",nchand,xlowd,xhighd);
	
	TH1F * wd1[4] = {w1d1,w2d1,w3d1,w4d1};

// Geometry Display

	TH2S * wg  = new TH2S("wg","WA98", 130, -65, 65, 130, -65, 65);

	TH2S * pg  = new TH2S("pg","Phobos", 10, -5, 5, 10, -5, 5);

// Energy Deposited
	
	TH1F * t1e = new TH1F("t1e","Telescope 1 Energy",stele[0],.5,stele[0]+.5);
	TH1F * t2e = new TH1F("t2e","Telescope 1 Energy",stele[1],.5,stele[1]+.5);
	TH1F * t3e = new TH1F("t3e","Telescope 1 Energy",stele[2],.5,stele[2]+.5);
	TH1F * t4e = new TH1F("t4e","Telescope 1 Energy",stele[3],.5,stele[3]+.5);
	TH1F * t5e = new TH1F("t5e","Telescope 1 Energy",stele[4],.5,stele[4]+.5);
	TH1F * t6e = new TH1F("t6e","Telescope 1 Energy",stele[5],.5,stele[5]+.5);

	TH1F * te[6] = {t1e,t2e,t3e,t4e,t5e,t6e};

	TH1F * w1e = new TH1F("w1e","WA98 1 Energy",swa98[0],.5,swa98[0]+.5);
	TH1F * w2e = new TH1F("w2e","WA98 2 Energy",swa98[1],.5,swa98[1]+.5);
	TH1F * w3e = new TH1F("w3e","WA98 3 Energy",swa98[2],.5,swa98[2]+.5);	
	TH1F * w4e = new TH1F("w4e","WA98 4 Energy",swa98[3],.5,swa98[3]+.5);

	TH1F * we[4] = {w1e,w2e,w3e,w4e};

	TH1F * p1e = new TH1F("p1e","Phobos 1 Energy",sphob[0],.5,sphob[0]+.5);

	TH1F * pe[1] = {p1e};

// Hit distribution	

	TH1S * t1h = new TH1S("t1h","Telescope 1 Hits",stele[0],.5,stele[0]+.5);
	TH1S * t2h = new TH1S("t2h","Telescope 2 Hits",stele[1],.5,stele[1]+.5);
	TH1S * t3h = new TH1S("t3h","Telescope 3 Hits",stele[2],.5,stele[2]+.5);
	TH1S * t4h = new TH1S("t4h","Telescope 4 Hits",stele[3],.5,stele[3]+.5);
	TH1S * t5h = new TH1S("t5h","Telescope 5 Hits",stele[4],.5,stele[4]+.5);
	TH1S * t6h = new TH1S("t6h","Telescope 6 Hits",stele[5],.5,stele[5]+.5);

	TH1S * th[6] = {t1h,t2h,t3h,t4h,t5h,t6h};

	TH1S * p1h = new TH1S("p1h","Phobos 1 Hits",sphob[0],.5,sphob[0]+.5);

	TH1S * ph[1] = {p1h};

	TH1S * w1h = new TH1S("w1h","WA98 1 Hits",swa98[0],.5,swa98[0]+.5);
	TH1S * w2h = new TH1S("w2h","WA98 2 Hits",swa98[1],.5,swa98[1]+.5);
	TH1S * w3h = new TH1S("w3h","WA98 3 Hits",swa98[2],.5,swa98[2]+.5);
	TH1S * w4h = new TH1S("w4h","WA98 4 Hits",swa98[3],.5,swa98[3]+.5);

	TH1S * wh[4] = {w1h,w2h,w3h,w4h};

// Multiplicity

	TH1S * t1m = new TH1S("t1m","Telescope 1 multiplicity",nchanm,xlowm,xhighm);
	TH1S * t2m = new TH1S("t2m","Telescope 2 multiplicity",nchanm,xlowm,xhighm);
	TH1S * t3m = new TH1S("t3m","Telescope 3 multiplicity",nchanm,xlowm,xhighm);
	TH1S * t4m = new TH1S("t4m","Telescope 4 multiplicity",nchanm,xlowm,xhighm);
	TH1S * t5m = new TH1S("t5m","Telescope 5 multiplicity",nchanm,xlowm,xhighm);
	TH1S * t6m = new TH1S("t6m","Telescope 6 multiplicity",nchanm,xlowm,xhighm);

	TH1S * tm[6] = {t1m,t2m,t3m,t4m,t5m,t6m};

	TH1S * p1m = new TH1S ("p1m","Phobos 1 multiplicity",sphob[0],0,sphob[0]+1);

	TH1S * pm[1] = {p1m};

	TH1S * w1m = new TH1S("w1m","WA98 1 multiplicity",nchanm,xlowm,xhighm);
	TH1S * w2m = new TH1S("w2m","WA98 2 multiplicity",nchanm,xlowm,xhighm);
	TH1S * w3m = new TH1S("w3m","WA98 3 multiplicity",nchanm,xlowm,xhighm);
	TH1S * w4m = new TH1S("w4m","WA98 4 multiplicity",nchanm,xlowm,xhighm);
	TH1S * w5m = new TH1S("w5m","WA98 5 multiplicity",nchanm,xlowm,xhighm);
	TH1S * w6m = new TH1S("w6m","WA98 6 multiplicity",nchanm,xlowm,xhighm);

	TH1S * wm[4] = {w1m,w2m,w3m,w4m};

// Telescope Tracking

	TH2S * t12 = new TH2S("t12","Tele x1 vs x2",stele[0],0,stele[0],stele[1],0,stele[1]);
    TH2S * t16 = new TH2S("t16","Tele x1 vs x6",stele[0],0,stele[0],stele[5],0,stele[5]);
    TH2S * t26 = new TH2S("t26","Tele x2 vs x6",stele[1],0,stele[1],stele[5],0,stele[5]);
    TH2S * t53 = new TH2S("t53","Tele y5 vs y3",stele[4],0,stele[4],stele[2],0,stele[2]);
    TH2S * t54 = new TH2S("t54","Tele y5 vs y4",stele[4],0,stele[4],stele[3],0,stele[3]);
    TH2S * t34 = new TH2S("t34","Tele y3 vs y4",stele[2],0,stele[2],stele[3],0,stele[3]);
    TH2S * t15 = new TH2S("t15","Tele x1 vs y5",stele[0],0,stele[0],stele[4],0,stele[4]);
	TH2S * t23 = new TH2S("t23","Tele x2 vs y3",stele[1],0,stele[1],stele[2],0,stele[2]);

	cout << "Histograms Booked" << endl;

// Read pedestal Files
	phob->ReadPedFile(pedphob);
    tele->ReadPedFile(pedtele);
	wa98->ReadPedFile(pedwa98);

// Open File
	TPhTestData * data = new TPhTestData(runfile);

	Int_t ndat = 0;
	Int_t nerr = 0;
	Int_t nped = 0;
	Int_t ncal = 0;
    Int_t loop = 0;

	TPhDetHit   * hit;
	TPhPad      * pad;
	TObjArray   * objarr;
	TIterator   * iter;

	if (data->IsOpen())
	{
		cout << "Data File is open" << endl;
// Main Loop


        for (loop = 1; !(data->IsEOF()) &&  loop != nevent+1; loop++)
//           for (loop = 1; loop < 501; loop++)
		{
// Read Next Event

			data->GetNextEvent(event);
	 
// Chose only Data Type
   	        Short_t dtype;
            dtype = event->GetDataType();

			if (!(loop%100))
		    {
				cout << loop << endl;
		    }
			if (dtype == 1) 
	        {
// Fill Data histograms
				ndat++;
		        event->FillData();
		        for (x = 0; x < ntele; x++)
				{
					nhit = dtele[x]->GetNDetHits();
	                tm[x]->Fill(nhit);
          	        for (i = 0; i < stele[x]; i++)
					{
						ped = dtele[x]->GetPed(i);
						raw = dtele[x]->GetRaw(i);
						sub = raw - ped;
						tp[x]->Fill(i+1,raw);
						td[x]->Fill(i+1,sub);
						td1[x]->Fill(sub);
						if (sub > ctele[x])
						{
							th[x]->Fill(i+1);
							te[x]->Fill(i+1,sub);
  						}
					}
				}
				for (x = 0; x < nwa98; x++)
				{
					nhit = dwa98[x]->GetNDetHits();
					wm[x]->Fill(nhit);
					objarr = dwa98[x]->GetDetHitList();
					if(objarr)
					{
						iter = objarr->MakeIterator();
						hit = (TPhDetHit *) iter->Next();
						{			
							pad = hit->GetPad();
							wg->Fill(pad->GetXcentP(),pad->GetYcentP());
							
							hit = (TPhDetHit *) iter->Next();
						}
						delete iter;
					}

        			for (i = 0; i < swa98[x]; i++)
					{
 						ped = dwa98[x]->GetPed(i);
						raw = dwa98[x]->GetRaw(i);
						sub = raw - ped;
						wp[x]->Fill(i+1,raw);
						wd[x]->Fill(i+1,sub);
						wd1[x]->Fill(sub);
						if (sub > cwa98[x])
						{
							wh[x]->Fill(i+1);
							we[x]->Fill(i+1,sub);
  						}
					}
				}
				for (x = 0; x < nphob; x++)
				{
					nhit = dphob[x]->GetNDetHits();
					pm[x]->Fill(nhit);
					objarr = dphob[x]->GetDetHitList();
					if(objarr)
					{
						iter = objarr->MakeIterator();
						hit = (TPhDetHit *) iter->Next();
						{			
							pad = hit->GetPad();
							pg->Fill(pad->GetXcentP(),pad->GetYcentP());
							
							hit = (TPhDetHit *) iter->Next();
						}
						delete iter;
					}
         			for (i = 0; i < sphob[x]; i++)
					{
						ped = dphob[x]->GetPed(i);
						raw = dphob[x]->GetRaw(i);
						sub = raw - ped;
						pp[x]->Fill(i+1,raw);
						pd[x]->Fill(i+1,sub);
						pd1[x]->Fill(sub);
						if (sub > cphob[x])
						{
							ph[x]->Fill(i+1);
							pe[x]->Fill(i+1,sub);
  						}
					}
				}
				// Telescope tracking
				Int_t a;
				Int_t b;
				TPhDetHit * ahit;
				TPhDetHit * bhit;
				TObjArray * aObj;
				TObjArray * bObj;
				TIterator * aIter;
				TIterator * bIter;

	// 12
				a = 0;
				b = 1;
				aObj = dtele[a]->GetDetHitList();
				bObj = dtele[b]->GetDetHitList();
				if(aObj && bObj)
				{
					aIter = aObj->MakeIterator();
					bIter = bObj->MakeIterator();
					ahit = (TPhDetHit *) aIter->Next();
					while (ahit)
					{
						bIter->Reset();
						bhit = (TPhDetHit *) bIter->Next();
						while (bhit)
						{
							t12->Fill(ahit->GetID(),bhit->GetID());
							bhit = (TPhDetHit *) bIter->Next();
						}
						ahit = (TPhDetHit *) aIter->Next();
					}
					delete aIter;
					delete bIter;
				}

	// 16
				a = 0;
				b = 5;
				aObj = dtele[a]->GetDetHitList();
				bObj = dtele[b]->GetDetHitList();
				if(aObj && bObj)
				{
					aIter = aObj->MakeIterator();
					bIter = bObj->MakeIterator();
					ahit = (TPhDetHit *) aIter->Next();
					while (ahit)
					{
						bIter->Reset();
						bhit = (TPhDetHit *) bIter->Next();
						while (bhit)
						{
							t16->Fill(ahit->GetID(),bhit->GetID());
							bhit = (TPhDetHit *) bIter->Next();
						}
						ahit = (TPhDetHit *) aIter->Next();
					}
					delete aIter;
					delete bIter;
				}

	// 26
				a = 1;
				b = 5;
				aObj = dtele[a]->GetDetHitList();
				bObj = dtele[b]->GetDetHitList();
				if(aObj && bObj)
				{
					aIter = aObj->MakeIterator();
					bIter = bObj->MakeIterator();
					ahit = (TPhDetHit *) aIter->Next();
					while (ahit)
					{
						bIter->Reset();
						bhit = (TPhDetHit *) bIter->Next();
						while (bhit)
						{
							t26->Fill(ahit->GetID(),bhit->GetID());
							bhit = (TPhDetHit *) bIter->Next();
						}
						ahit = (TPhDetHit *) aIter->Next();
					}
					delete aIter;
					delete bIter;
				}

	// 53
				a = 4;
				b = 2;
				aObj = dtele[a]->GetDetHitList();
				bObj = dtele[b]->GetDetHitList();
				if(aObj && bObj)
				{
					aIter = aObj->MakeIterator();
					bIter = bObj->MakeIterator();
					ahit = (TPhDetHit *) aIter->Next();
					while (ahit)
					{
						bIter->Reset();
						bhit = (TPhDetHit *) bIter->Next();
						while (bhit)
						{
							t53->Fill(ahit->GetID(),bhit->GetID());
							bhit = (TPhDetHit *) bIter->Next();
						}
						ahit = (TPhDetHit *) aIter->Next();
					}
					delete aIter;
					delete bIter;
				}

	// 54
				a = 4;
				b = 3;
				aObj = dtele[a]->GetDetHitList();
				bObj = dtele[b]->GetDetHitList();
				if(aObj && bObj)
				{
					aIter = aObj->MakeIterator();
					bIter = bObj->MakeIterator();
					ahit = (TPhDetHit *) aIter->Next();
					while (ahit)
					{
						bIter->Reset();
						bhit = (TPhDetHit *) bIter->Next();
						while (bhit)
						{
							t54->Fill(ahit->GetID(),bhit->GetID());
							bhit = (TPhDetHit *) bIter->Next();
						}
						ahit = (TPhDetHit *) aIter->Next();
					}
					delete aIter;
					delete bIter;
				}

	// 34
				a = 2;
				b = 3;
				aObj = dtele[a]->GetDetHitList();
				bObj = dtele[b]->GetDetHitList();
				if(aObj && bObj)
				{
					aIter = aObj->MakeIterator();
					bIter = bObj->MakeIterator();
					ahit = (TPhDetHit *) aIter->Next();
					while (ahit)
					{
						bIter->Reset();
						bhit = (TPhDetHit *) bIter->Next();
						while (bhit)
						{
							t34->Fill(ahit->GetID(),bhit->GetID());
							bhit = (TPhDetHit *) bIter->Next();
						}
						ahit = (TPhDetHit *) aIter->Next();
					}
					delete aIter;
					delete bIter;
				}

	// 15
				a = 0;
				b = 4;
				aObj = dtele[a]->GetDetHitList();
				bObj = dtele[b]->GetDetHitList();
				if(aObj && bObj)
				{
					aIter = aObj->MakeIterator();
					bIter = bObj->MakeIterator();
					ahit = (TPhDetHit *) aIter->Next();
					while (ahit)
					{
						bIter->Reset();
						bhit = (TPhDetHit *) bIter->Next();
						while (bhit)
						{
							t15->Fill(ahit->GetID(),bhit->GetID());
							bhit = (TPhDetHit *) bIter->Next();
						}
						ahit = (TPhDetHit *) aIter->Next();
					}
					delete aIter;
					delete bIter;
				}

	// 23
				a = 1;
				b = 2;
				aObj = dtele[a]->GetDetHitList();
				bObj = dtele[b]->GetDetHitList();
				if(aObj && bObj)
				{
					aIter = aObj->MakeIterator();
					bIter = bObj->MakeIterator();
					ahit = (TPhDetHit *) aIter->Next();
					while (ahit)
					{
						bIter->Reset();
						bhit = (TPhDetHit *) bIter->Next();
						while (bhit)
						{
							t23->Fill(ahit->GetID(),bhit->GetID());
							bhit = (TPhDetHit *) bIter->Next();
						}
						ahit = (TPhDetHit *) aIter->Next();
					}
					delete aIter;
					delete bIter;
				}

			}
			else if (dtype == 2)
			{
				nped++;
		        for (x = 0; x < ntele; x++)
				{
          	        for (i = 0; i < stele[x]; i++)
					{
						raw = dtele[x]->GetRaw(i);
						tpp[x]->Fill(i+1,raw);
					}
				}
				for (x = 0; x < nphob; x++)
				{
          	        for (i = 0; i < sphob[x]; i++)
					{
						raw = dphob[x]->GetRaw(i);
						ppp[x]->Fill(i+1,raw);
					}
				}
				for (x = 0; x < nwa98; x++)
				{
          	        for (i = 0; i < swa98[x]; i++)
					{
						raw = dwa98[x]->GetRaw(i);
						wpp[x]->Fill(i+1,raw);
					}
				}
            }
            else if (dtype == 3) 
            {
				ncal++;
            }
            else 
			{
				nerr++;
			}
		}
	}
	else
	{
		cout << "Input File not present" << endl;      
	}

// Write cout event statistics

	cout << "Number of Events      = " << loop-1 << endl;;
	cout << "Number of Data        = " << ndat << endl;
	cout << "Number of pedestals   = " << nped << endl;
	cout << "Number of Calibration = " << ncal << endl;
    cout << "Number of Errors      = " << nerr << endl;

// Write out histograms
	
	cerr << endl;
	cerr << endl;
	cerr << "An upcoming crash is expected after the file write" << endl;
	cerr << "Just click close, close, then close the window" << endl;
	cerr << endl;

	hfile->Write();
	hfile->Close();

// We Be Done
	cerr << "File is done" << endl;
	return 0;

}