
#ifndef _PhDetFamily_INCLUDED_
#define _PhDetFamily_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhDetFamily                                                   //
//                                                                   //
//    Phobos DetFamily base class                                    //
//                                                                   // 
//    Specific DetFamily types inhererit from this                   //
//                                                                   //
//    Autheor : Christopher Conner (cmc@uic.edu)                     //
//    Created : 04/23/97                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// General Classes

#include <iostream.h>

// Root Classes

#include <TNamed.h>
#include <TObjArray.h>

// Phobos Classes

class TPhDetFamily;
#include "TPhDetector.h"


class TPhDetFamily : public TNamed {

public:
    TPhDetFamily();
    
    virtual ~TPhDetFamily();

    virtual Bool_t Draw(Int_t color = -1);
    virtual Bool_t DrawPoint(Int_t color = -1);

    virtual TObjArray *    GetDetectorList()   { return fDetectorList; }

    virtual void      ResetDetectorIterator();
	virtual void      SetupDetectorIterator(Bool_t dir=kIterForward);
    
 	virtual TPhDetector *    GetNextDetector();

	virtual TPhDetector *    GetDetector(Int_t ID);
    
    virtual Int_t     GetNDetectors();

    virtual void      AddDetector(TPhDetector *); 
	virtual Int_t     DropDetector(Int_t ID); 
    virtual Int_t     DropDetector(TPhDetector *);
	virtual void      DropAllHits();

	virtual Int_t  GetID()         { return fID;    }
	virtual Int_t  GetDetTypeID()  { return fDetTypeID;}
	
	virtual void     SetEvent(TPhEvent * x) { fEvent = x;}

	virtual Bool_t     ReadGeometry(char *fname1, char *fname2) = 0;
    virtual Bool_t     ReadPedFile(char * fname1 = NULL);
    virtual Bool_t     WritePedFile(char * fname1 = NULL);
	virtual void     FillData();
	virtual void     CalcPed();
	virtual void     AddPed();

protected:
    Int_t fID;           // Detector ID
    Int_t fDetTypeID;    // Detector type ID
	TPhEvent * fEvent;        // Pointer to Event;

    
    TObjArray* fDetectorList;      //  ObjArray of all detectors in Family

	TIterator* fDetectorIterator;  // Iterator for detector list


public:

    ClassDef(TPhDetFamily,1) // PHOBOS DetFamily base class

};

#endif




