
#include "TPhDetector.h"

//#define DEBUG
//#define DEBUGFILL
//#define DEBUGA
//#define DEBUGGDH

ClassImp(TPhDetector)


////////////////////////////////////////////////////////////
//
// TPhDetector is a PHOBOS base data class providing storage 
// for all the hits on one detector plane.  All detectors
// will inherit from this class.
//
////////////////////////////////////////////////////////////

TPhDetector::TPhDetector()
{
// Constructor. Set lists to NULL; Sets Iterators to NULL;

    fDetHitList = NULL;
	fPadList =    NULL;
	fDetHitIterator = NULL;
	fPadIterator    = NULL;
	fRaw            = NULL;
	fPed            = NULL;
	fXcent = 0.;
	fYcent = 0.;
	fZcent = 0.;
	fThX   = 90.;
    fPhX   = 0.;
	fThY   = 90.;
	fPhY   = 90.;
	fThZ   = 0.;
	fPhZ   = 0.;
	fFrac  = .01;
	fNPedEvent = 0;
	fHitIDMin  = 0;
	fHitIDMax  = 0;
	fCut = 10;
}

TPhDetector::~TPhDetector()
{
// Destructor. Free all memory, destroying also the hit objects
//  in the fHitList and fCalHitList arrays and their Iterators

	if(fDetHitIterator) delete fDetHitIterator;
	fDetHitIterator = NULL;
    if(fDetHitList)
    {
        fDetHitList->Delete();
        delete fDetHitList;
		fDetHitList = NULL;
    }
	if (fPadList)
	{
		fPadList->Delete();
		delete fPadList;
		fPadList = NULL;
	}
	if (fRaw)
	{
		delete fRaw;
		fRaw = NULL;
	}
	if (fPed)
	{
		delete fPed;
	    fPed = NULL;
	}
}

void TPhDetector::AddDetHit(TPhDetHit *Hit)
{
// Add a pointer to a TPhDetHit object to the fHitList
// array. For the first Det hit, fHitList is allocated 
// here.

   if(!fDetHitList)
       fDetHitList = new TObjArray();
   fDetHitList->Add(Hit);
}

void TPhDetector::SetupRawData(Int_t nchan)
{
// Make storage for Raw Data
	if (fRaw)
	{
#ifdef DEBUG
		cout << "Deleting Raw Data" << endl;
#endif
			delete fRaw;
		cerr << "Raw Data Already setup" << endl;
			fRaw = NULL;
			
	}
	
	fRaw = new Short_t[nchan];
	for (Int_t i = 0; i< nchan; i++)
	{
		fRaw[i] = -1;
	}
	
}

void TPhDetector::SetupPedData(Int_t nchan)
{
// Make storage for pedestal Data
	if (fPed)
	{
#ifdef DEBUG
		cout << "Deleting Ped Data" << endl;
#endif
		cerr << "pedestal Data Already setup" << endl;
		delete fPed;
		fPed = NULL;
			
	}
	
	fPed = new Float_t[nchan];
	for (Int_t i = 0; i < nchan; i++)
	{
		fPed[i] = 0;
	}

	
}

Int_t TPhDetector::DropDetHit(TPhDetHit* hit)
{
   // Remove hit from array.
   // if deleting from wrong array return -1;

   if (!hit) 
   {
#ifdef DEBUG
	   cout << "No hit given to delete" << endl;
#endif
	   return 0; // no hit nothing to delete no error
   }
   if (!fDetHitList)
   {
	   
#ifdef DEBUG
	   cout << "Not list to delete from" << endl;
#endif
	   return -1; // no list to delete from;
   }
   else
   {
	if ((fDetHitList->IndexOf(hit) - 
		 fDetHitList->LowerBound()) == -1)
	{
#ifdef DEBUG
		cout << "Hit not in list" << endl;
#endif

		return -1; // object not in list
	}
	else
	{
	   fDetHitList->Remove(hit);
#ifdef DEBUG
	   cout << "Hit Deleted" << endl;
#endif
       delete hit;
	   hit = NULL;
	   return 0;
	}

   }

}

Int_t TPhDetector::DropPad(TPhPad * pad)
{
   // Remove pad from array.
   // if deleting from wrong array return -1;

   if (!pad) return 0; // no pad nothing to delete no error

   if (!fPadList)
   {
	   return -1; // no list to delete from;
   }
   else
   {
	if ((fPadList->IndexOf(pad) - 
		 fPadList->LowerBound()) == -1)
	{
		return -1; // object not in list
	}
	else
	{
	   fPadList->Remove(pad);
       delete pad;
	   pad = NULL;
	   return 0;
	}

   }

}


Int_t TPhDetector::DropDetHit(Int_t id)
{
   // Remove hit from array.
   // if hit doesn't exist give error
  TPhDetHit * hit = GetDetHit(id);
  if (hit)
  {
	  return DropDetHit(hit);
  }
  else
  { 
	  return -1;
  }
}

Int_t TPhDetector::DropPad(Int_t id)
{
   // Remove pad from array.
   // if hit doesn't exist give error
  TPhPad * pad = GetPad(id);
  if (pad)
  {
	  return DropPad(pad);
  }
  else
  { 
	  return -1;
  }
}


Int_t TPhDetector::GetNDetHits()
{
// Return number of Raw hits in the detector, if any.

    if(fDetHitList)
        return fDetHitList->GetEntries();
    else
        return 0;
}


TPhDetHit* TPhDetector::GetNextDetHit()
{
	// Iterator over all hits in the array.  
	// Uses fDetHitIterator
	// Cannot be nested
	// Use GetDetHitList and use TObjArrayIterator class. for nesting
	// Returns zero if no more hits in array

    if(fDetHitList)
    {
#ifdef DEBUGGDH
		cout << "Have det hit list" << endl;
#endif
      if(!fDetHitIterator)
	  {
#ifdef DEBUGGDH
		cout << "Making Iterator" << endl;
#endif

		  SetupDetHitIterator();
		  ResetDetHitIterator();
	  }
      return (TPhDetHit *) fDetHitIterator->Next();
    }
    else
    {
#ifdef DEBUGGDH
		cout << "No more hits" << endl;
#endif


        return 0;
    }
}

TPhPad* TPhDetector::GetNextPad()
{
	// Iterator over all pads in the array.  
	// Uses fPadCounter
	// Cannot be nested
	// Use GetPadList and use TObjArrayIterator class. for nesting
	// Returns zero if no more hits in array

    if(fPadList)
    {
      if(!fPadIterator)
	  {
		  SetupPadIterator();
		  ResetPadIterator();
	  }
      return (TPhPad *) fPadIterator->Next();
    }
    else
    {
        return 0;
    }
}

void TPhDetector::ResetDetHitIterator()
{
	// Resets the Det Hit list Iterator.
	// If not present it creates a new Iterator


	if(fDetHitList)
	{
      if(!fDetHitIterator)
	  {
        fDetHitIterator = fDetHitList->MakeIterator();
	  }
	  else
	  {
        fDetHitIterator->Reset();
	  }
	}
}

void TPhDetector::ResetPadIterator()
{
	// Resets the Pad list Iterator.
	// If not present it creates a new Iterator


	if(fPadList)
	{
      if(!fPadIterator)
	  {
        fPadIterator = fPadList->MakeIterator();
	  }
	  else
	  {
        fPadIterator->Reset();
	  }
	}
}

void TPhDetector::SetupDetHitIterator(Bool_t dir)
{
	// Creates a new Det Hit list Iterator.
	// If one already present it deletes it and makes a new one	
	// By default the Iteratoration direction is kIteratorForward. 
    // To go backward use kIteratorBackward.


	if(fDetHitIterator)
	{
		delete fDetHitIterator;
		fDetHitIterator = NULL;
	}
	if(fDetHitList)
	{
        fDetHitIterator = fDetHitList->MakeIterator(dir);
	}
}

void TPhDetector::SetupPadIterator(Bool_t dir)
{
	// Creates a new Pad list Iterator.
	// If one already present it deletes it and makes a new one	
	// By default the Iteratoration direction is kIteratorForward. 
    // To go backward use kIteratorBackward.


	if(fPadIterator)
	{
		delete fPadIterator;
		fPadIterator = NULL;
	}
	if(fPadList)
	{
        fPadIterator = fPadList->MakeIterator(dir);
	}
}
TPhDetHit *TPhDetector::GetDetHit(Int_t ID)
{
// Get the Det hit with given ID number
// Resets the DetHitIterator
	
    if(fDetHitList)
    {
       ResetDetHitIterator();
	   TPhDetHit* hit = GetNextDetHit();
	   while (hit && hit->GetID() != ID)
	   {
		   hit = GetNextDetHit();
	   }
	   return hit;
	   
    }
    else
    {
        return 0;
    }
}

Bool_t TPhDetector::Draw(Int_t color)
{
	// Draw the Detector Geometry as pads
    TPhPad * pad;  

	ResetPadIterator();
    while(pad = GetNextPad())
    {
#ifdef DEBUG2
		if (pad->GetID() == 1)
		{
	    cout << "Drawing Pad #" << pad->GetID() << endl; 
		}
#endif
		pad->Draw(color);
    }
    return kTRUE;
}

Bool_t TPhDetector::DrawPoint(Int_t color)
{
	// Draw the Detector Geometry as points
    TPhPad * pad;  

	ResetPadIterator();
    while(pad = GetNextPad())
    {
#ifdef DEBUG
		if (pad->GetID() == 1)
		{
	    cout << "Drawing Pad #" << pad->GetID() << endl; 
		}
#endif
		pad->DrawPoint(color);
    }
    return kTRUE;
}

void  TPhDetector::DropAllHits()
{
	// Drop all DetHits and set Raw Data to -1
#ifdef DEBUG
		cout << "Deleting detector hits" << endl;
#endif
	if (fDetHitList)
	{
		DeleteDetHits();
	}
#ifdef DEBUG
		cout << "Setting raw values to -1" << endl;
#endif

	if (fRaw)
	{
		for (Int_t i = 0; i << GetNChannels(); i++)
		{
			fRaw[i] = -1;
		}
	}
	
}

void TPhDetector::SetPedData   (Float_t *  x)
{
	// Store the Given pedestal Values
		if (fPed)
		{
			for (Int_t i = 0; i < GetNChannels(); i++)
			{
				fPed[i] = x[i];
			}
		}
		else
		{
			cerr << "No place to store pedestals" << endl;
		}

}

void TPhDetector::SetRawData   (Short_t *  x)
{
		if (fRaw)
		{
#ifdef DEBUG
			cout << "Setting Raw data ";
#endif
			for (Int_t i = 0; i < GetNChannels(); i++)
			{
#ifdef DEBUG
				cout << x[i] << ':';
#endif
				fRaw[i] = x[i];
			}
#ifdef DEBUG
			cout << endl;
#endif
		}
		else
		{
			cerr << "No place to store data" << endl;
		}

}

void TPhDetector::AddPed()
{
	// Calculate a running pedestal Value
	// all events have same weight
	if (fPed)
	{
		for (Int_t i = 0; i < GetNChannels(); i++)
		{
			fPed[i] = (fNPedEvent * fPed[i] + fRaw[i])/(fNPedEvent+1);
		}
		
	}
	fNPedEvent++;
}

void TPhDetector::CalcPed()
{
// Calculate a running pedestal Value.
// Only last 1/fFrac events are kept.
// 1/fFrac = 100 by default.
	fNPedEvent++;
	if (fPed && fRaw)
	{
		if (!(fFrac >0.) || fNPedEvent < 2)
		{
			for (Int_t i = 0; i < GetNChannels(); i++)
			{
				fPed[i] = fRaw[i];
			}
		}
		else
		{
			// Less than 1/fFrac events
			Float_t frac;
			if (1/fNPedEvent > fFrac)
			{
				frac = 1./fNPedEvent;
			}
			else
			{
				frac = fFrac;
			}
			for (Int_t i = 0; i < GetNChannels(); i++)
			{
				fPed[i] = (fPed[i] + fRaw[i]*frac)/(Float_t(1.)+frac);
		
			}
		}
	}
	fNPedEvent++;
}

void TPhDetector::FillData()
{
	// Fill the Real data.
	// Take raw data, subtract pedestal, subtract the cut
	// store in DetHits
#ifdef DEBUGFILL
	cout << "Filling Data" << endl;
#endif
	if (fRaw && fPed)
	{
#ifdef DEBUGFILL
	cout << "Pointers are ok" << endl;
#endif

		for (Int_t i = 0; i < GetNChannels(); i++)
		{
#ifdef DEBUGFILL
	cout << "In Loop " << i << endl;
#endif

			Float_t val = fRaw[i] - fPed[i] - fCut;
			if (val > 0)
			{
#ifdef DEBUGFILL
	cout << "Adding point " << i << endl;
#endif
				TPhPad * pad = GetPad(i+fPadList->LowerBound());
				if (!pad)
				{
					cerr << "Pad not found for id #" << (i+fPadList->LowerBound()) <<endl;
				}
				else if (pad->IsActive())
				{
					TPhDetHit * hit = new TPhDetHit();
					hit->SetPad(pad);
					hit->SetValue(val+fCut);
					hit->SetID(i);
					hit->SetDetector(this);
					AddDetHit(hit);
				}
				else
				{
					cerr << "Pad not active, data point skipped" << endl;
				}
			}
		}
#ifdef DEBUGFILL
	cout << "Done With Fill " << i << endl;
#endif

	}
	else
	{
		cerr << "Data storage not set up" << endl;
	}
}

void TPhDetector::ZeroPed()
{
	// Set all pedestals to Zero
	if(fPed) 
	{
		for (Int_t i = 0; i < GetNChannels(); i++)
		{
			fPed[i] = Float_t(0);
		}
	}
}

void TPhDetector::SetupPadList(Int_t size, Int_t lowid)
{
	// Setup storage for the Pads (Geometry)
	// Only call once with number of channels
	if (fPadList)
	{
#ifdef DEBUG
		cout << "deleting pad list " << endl;
#endif
		DeletePads();
	}
	fPadList = new TObjArray(size,lowid);
}
//
void TPhDetector::AddPad(TPhPad * pad, Int_t id)
{
	// Add a Pad to the list
	// Must call SetupPadList first
	if (!fPadList)
	{
		cerr << "Pad list not initialized" << endl;
		SetupPadList(1,id);
	}
	fPadList->AddAtAndExpand(pad, id);
	pad->SetID(fPadList->IndexOf(pad));
}


TPhPad * TPhDetector::GetPad(Int_t ID)
{
// Get the Pad with given ID number
// return 0 if no pad

	return (TPhPad *)(fPadList->At(ID));
    
}

void TPhDetector::SetupAllData(Int_t size, Int_t lowid)
{
	// Setup storage for Pads,RawData, and pedestals
	SetupPadList(size,lowid);
	SetupRawData(size);
	SetupPedData(size);
}

void TPhDetector::DeleteDetHits()
{
	// Delete all DetHits
	if (fDetHitList) 
	{
		fDetHitList->Delete(); 
		delete fDetHitList; 
		fDetHitList = NULL;
	}
}

void TPhDetector::DeletePads()
{
	// Delete All Pads
	if (fPadList) 
	{
#ifdef DEBUG
		cout << "Deleting Pads" << endl;
#endif

		fPadList->Delete(); 
		delete fPadList; 
		fPadList = NULL;
	}
}
