
#ifndef _PhDetector_INCLUDED_
#define _PhDetector_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhDetector                                                    //
//                                                                   //
//    Phobos Detector base class                                     //
//                                                                   // 
//    Specific detector types inhererit from this                    //
//                                                                   //
//    Author  : Christopher Conner (cmc@uic.edu)                     //
//    Created : 04/21/97                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// General Classes

#include <iostream.h>

// Root Classes

#include <TNamed.h>
#include <TObjArray.h>

// Phobos Classes

class TPhDetector;

#include "TPhEvent.h"
#include "TPhDetHit.h"
#include "TPhPad.h"
#include "TPhPed.h"

class TPhDetector : public TNamed {

public:
    TPhDetector();
    
    virtual ~TPhDetector();

    virtual Bool_t Draw(Int_t color=-1);
    virtual Bool_t DrawPoint(Int_t color=-1);

	virtual void AddPed();
	virtual void FillData();
	virtual void CalcPed();
	virtual void ZeroPed(); 
    virtual TObjArray*    GetDetHitList()   { return fDetHitList; }
    virtual TObjArray*    GetPadList()      { return fPadList;    }

    virtual void      ResetDetHitIterator();
	virtual void      ResetPadIterator();

	virtual void      SetupDetHitIterator(Bool_t dir=kIterForward);
    virtual void      SetupPadIterator(Bool_t dir=kIterForward);

    virtual TPhDetHit*    GetNextDetHit();
    virtual TPhPad*       GetNextPad();

	virtual TPhDetHit*    GetDetHit(Int_t ID);
	virtual TPhPad*       GetPad(Int_t ID);
    virtual Float_t       GetPed(Int_t ID) {return fPed[ID];}
	virtual Short_t       GetRaw(Int_t ID) {return fRaw[ID];}
    
    virtual Int_t     GetNDetHits();
	virtual Int_t     GetNPads() {return fPadList->GetEntriesFast();}

    virtual void      AddDetHit(TPhDetHit *Hit);
	virtual void      AddPad(TPhPad *, Int_t id);

    virtual Int_t     DropDetHit(Int_t ID); 
    virtual Int_t     DropPad(Int_t ID);

    virtual Int_t     DropDetHit(TPhDetHit * Hit); 
    virtual Int_t     DropPad(TPhPad * Pad);

	virtual void      DropAllHits();
    virtual void      DeleteDetHits();
    virtual void      DeletePads();

	virtual Int_t   GetDetID()      { return fDetID;    }
	virtual Int_t   GetDetTypeID()  { return fDetTypeID;}
    virtual Int_t   GetHitIDMin()   { return fPadList->LowerBound(); }
	virtual Int_t   GetHitIDMax()   { return fPadList->GetLast(); }
	virtual Int_t   GetNChannels()  { return fPadList->GetEntriesFast();}
    virtual Double_t   GetXcent()      { return fXcent;    }
	virtual Double_t   GetYcent()      { return fYcent;    }
    virtual Double_t   GetZcent()      { return fZcent;    }
	virtual Double_t   GetThX()        { return fThX;      }
    virtual Double_t   GetThY()        { return fThY;      }
	virtual Double_t   GetThZ()        { return fThZ;      }
	virtual Double_t   GetPhX()        { return fPhX;      }
    virtual Double_t   GetPhY()        { return fPhY;      }
	virtual Double_t   GetPhZ()        { return fPhZ;      }
	virtual Short_t *  GetRawData()    { return fRaw;      }
	virtual Float_t *  GetPedData()    { return fPed;      }  
	virtual Float_t    GetCut()        { return fCut;      }
	virtual Float_t    GetFrac()       { return fFrac;     }
	virtual Int_t      GetNPedEvent()  { return fNPedEvent;}

	virtual void     SetNPedEvent(Int_t x)  { fNPedEvent = x;}
	virtual void     SetEventPointer(TPhEvent * x) {fEventPointer = x;}
	virtual void     SetFrac(Float_t x)     { fFrac      = x;}
    virtual void     SetCut(Float_t x)      { fCut       = x;}
    virtual void     SetDetID     (Int_t x) { fDetID     = x;}
	virtual void     SetDetTypeID (Int_t x) { fDetTypeID = x;}
    virtual void     SetHitIDMin  (Int_t x) { fHitIDMin  = x;}
	virtual void     SetHitIDMax  (Int_t x) { fHitIDMax  = x;}
    virtual void     SetXcent     (Double_t x) { fXcent     = x;}
	virtual void     SetYcent     (Double_t x) { fYcent     = x;}
    virtual void     SetZcent     (Double_t x) { fZcent     = x;}
	virtual void     SetThX       (Double_t x) { fThX       = x;}
    virtual void     SetThY       (Double_t x) { fThY       = x;}
	virtual void     SetThZ       (Double_t x) { fThZ       = x;}
	virtual void     SetPhX       (Double_t x) { fPhX       = x;}
    virtual void     SetPhY       (Double_t x) { fPhY       = x;}
	virtual void     SetPhZ       (Double_t x) { fPhZ       = x;}
	virtual void     SetRawData   (Short_t *  x);
	virtual void     SetPedData   (Float_t *  x);
	virtual void     SetPed(Int_t ID, Float_t x) {fPed[ID] = x;}
	virtual void     SetRaw(Int_t ID, Short_t x) {fRaw[ID] = x;}
    
    virtual void     SetupPadList (Int_t size, Int_t lowid=0);
	virtual void     SetupPedData (Int_t size);
	virtual void     SetupRawData (Int_t size);
	virtual void     SetupAllData (Int_t size, Int_t lowid=0);

protected:
    Int_t fDetID;       // Detector ID
    Int_t fDetTypeID;   // Detector type ID
	Int_t fHitIDMin;    // Minimum Hit ID 
	Int_t fHitIDMax;    // Maximum Hit ID

    Double_t fXcent;       // center of detector in X
    Double_t fYcent;       // center of detector in Y
    Double_t fZcent;       // center of detector in Z

	Double_t fThX;         // X projection of Theta ?
    Double_t fThY;         // Y projection of Theta ?
    Double_t fThZ;         // Z projection of Theta ?
    Double_t fPhX;         // X projection of Phi ?
    Double_t fPhY;         // Y projection of Phi ?
    Double_t fPhZ;         // Z projection of Phi ?

    Short_t * fRaw;          // Raw Data
	Float_t * fPed;          // Ped Data
	Float_t   fCut;          // Cut for data
	Float_t   fFrac;         // Fraction for Pedestal update.
    
    TObjArray * fDetHitList;      //  ObjArray of detector hits on one plane
    TObjArray * fPadList;         //  ObjArray of pads

	TIterator * fDetHitIterator;  // Iterator for Det Hit List
    TIterator * fPadIterator;     // Iterator for Pad List

	TPhEvent  * fEventPointer;    // Pointer to event;
	Int_t fNPedEvent;			  // Current number of Pedestal Events	

public:

    ClassDef(TPhDetector,1) // PHOBOS detector base class

};

#endif




