#ifndef _PhEvent_INCLUDED_
#define _PhEvent_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhEvent                                                       //
//                                                                   //
//    Phobos Event base class                                        //
//                                                                   // 
//    Specific Event types inhererit from this                       //
//                                                                   //
//    Autheor : Christopher Conner (cmc@uic.edu)                     //
//    Created : 04/23/97                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// Root Classes

#include <TNamed.h>
#include <TObjArray.h>

// Phobos Classes

class TPhEvent;
#include "TPhDetFamily.h"


enum TPhEvent_t {kSIGNAL     =1, kpedestal=2,
	             kCALIBRATION=3, kERROR   =4}; 
	

class TPhEvent : public TNamed {

public:

    TPhEvent();
    
    virtual ~TPhEvent();
    
	virtual TObjArray*    GetFamilyList()   { return fFamilyList; }

    virtual void      ResetFamilyIterator();
	virtual void      SetupFamilyIterator(Bool_t dir=kIterForward);
    
 	virtual TPhDetFamily *    GetNextFamily();

	virtual TPhDetFamily *    GetFamily(Int_t ID);
    
    virtual Int_t     GetNFamily();
	virtual Int_t     GetNFamilyMembers(Int_t TypeID);

    virtual void      AddFamily(TPhDetFamily *); 
	virtual Int_t     DropFamily(Int_t ID); 
    virtual Int_t     DropFamily(TPhDetFamily *);
	virtual void      DropAllHits();
	virtual void      FillData();
	virtual void      CalcPed();
	virtual void      AddPed();
	virtual Bool_t    Draw(Int_t color=-1);
	virtual Bool_t    DrawPoint(Int_t color=-1);

	virtual Int_t  GetSoftware()  { return fSoftware; }
    virtual Int_t  GetEventNumber(){ return fEvent;    }
    virtual Int_t  GetRun()       { return fRun;      }
    virtual Int_t  GetDate()      { return fDate;     }
    virtual Int_t  GetTime()      { return fTime;     }
    virtual TPhEvent_t  GetDataType()  { return fDataType; }

	virtual void   SetSoftware(Int_t x)  {fSoftware = x;}
    virtual void   SetEvent   (Int_t x)  {fEvent    = x;}
    virtual void   SetRun     (Int_t x)  {fRun      = x;}
    virtual void   SetDate    (Int_t x)  {fDate     = x;}
    virtual void   SetTime    (Int_t x)  {fTime     = x;}
    virtual void   SetDataType(TPhEvent_t x)  {fDataType = x;}

protected:

    TObjArray    *fFamilyList;    //  ObjArray of all Familys in Family

	TIterator * fFamilyIterator;  // Iterator for Family list

	Int_t fSoftware; // Sofware Version
	Int_t fEvent;    // Event Number
	Int_t fRun;      // Run Number
	Int_t fDate;     // Date (YYYYMMDD)
	Int_t fTime;     // time (HHMMSS)
	TPhEvent_t fDataType; // (kSIGNAL      =1, kpedestal=2, 
	                      //  kCALIBRATION =3, kERROR   =4)
	
	
public:

    ClassDef(TPhEvent,1) // PHOBOS Event base class

};

#endif




