#include <math.h>
#include <iostream.h>

#include "TPhPad.h"

//#define DEBUG

ClassImp(TPhPad)

/////////////////////////////////////////////////////////////
//
// TPhPad handles the management of geometry information 
// for one Pad.  
// It stores the center values in X,Y,Z
// It stores its width,height, and thickness
// It stores a ployline that contains its outline
//
/////////////////////////////////////////////////////////////

TPhPad::TPhPad() : fActive(kTRUE)
{
// Constructor for TPhPad. 
}

TPhPad::~TPhPad()
{
// Destructor for TPhPad. Delete PolyLine if it exists
	if (fPoly)
	{delete fPoly;}

}

void TPhPad::Draw(Int_t color)
{
// Drawing routine for one Pad. This is not standalone,
// but has to be called after creating a TView object

    if(!fPoly)
    {
        fPoly = new TPolyLine3D(5);
		for (Int_t num = 0; num < 4; num++)
		{
			fPoly->SetPoint(num,Float_t(fX[num]),Float_t(fY[num]),Float_t(fZ[num]));
		}
		fPoly->SetPoint(4,Float_t(fX[0]),Float_t(fY[0]),Float_t(fZ[0]));
		if (color < 1) color = fPlane;
        fPoly->SetLineColor(color);
	}
	

    fPoly->Draw();
#ifdef DEBUG
	if (fID == 1)
	{
    	cout << "Drawing Pad #" << fID << ' ';
		cout << ":" << fX[0];
		cout << ":" << fY[0];
		cout << ":" << fZ[0];
		cout << ":" << endl;
	}
#endif
}

void TPhPad::DrawPoint(Int_t color)
{
	if(!fPolyM)
    {
#ifdef DEBUG2
	cout << "Adding point  " << fXcent << ':' << fYcent << ':' << fZcent << endl;
#endif
        fPolyM = new TPolyMarker3D(1,1);
	    fPolyM->SetPoint(0,Float_t(fXcent),Float_t(fYcent),Float_t(fZcent));
	}
	if (color < 1) color = fPlane;
    fPolyM->SetMarkerColor(color);
    fPolyM->Draw();
#ifdef DEBUG
	if (fID == 1)
	{
    	cout << "Drawing Point #" << fID << ' ';
		cout << ":" << fX[0];
		cout << ":" << fY[0];
		cout << ":" << fZ[0];
		cout << ":" << endl;
	}
#endif
#ifdef DEBUG2
	cout << "Drawing point " << fXcent << ':' << fYcent << ':' << fZcent << endl;
#endif
}
