#ifndef _PhPad_INCLUDED_
#define _PhPad_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhPad                                                         //
//                                                                   //
//    Phobos event class                                             //
//                                                                   //
//    TPhPad manages the geometry for one pad                        //
//    All PadTypes inherit from this class                           //
//                                                                   //
//    Author  : Christopher Conner (cmc@uic.edu)                     //
//    Created : 04/27/97                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// Root Classes

#include <TObject.h>
#include <TPolyLine3D.h>
#include <TPolyMarker3D.h>

// Phobos Classes

class TPhPad : public TObject {

public:
    TPhPad();
    
    virtual ~TPhPad();

    virtual Int_t     GetID()          { return fID;        }
	virtual Int_t     GetPlane()       { return fPlane;     }
    virtual Int_t     GetFamily()      { return fFamily;    }
    virtual Double_t     GetXcent()       { return fXcent;     }
    virtual Double_t     GetYcent()       { return fYcent;     }
    virtual Double_t     GetZcent()       { return fZcent;     }
	virtual Double_t     GetXcentP()      { return fXcentP;    }
	virtual Double_t     GetYcentP()      { return fYcentP;    }
    virtual Double_t     GetWidth()       { return fWidth;     }
    virtual Double_t     GetHeight()      { return fHeight;    }
    virtual Double_t     GetThickness()   { return fThickness; }
	virtual Bool_t      IsActive()       { return fActive;    }
    virtual Double_t     GetCornersX0()   { return fX[0];      }
    virtual Double_t     GetCornersY0()   { return fY[0];      }
    virtual Double_t     GetCornersZ0()   { return fZ[0];      }
    virtual Double_t     GetCornersX1()   { return fX[1];      }
    virtual Double_t     GetCornersY1()   { return fY[1];      }
    virtual Double_t     GetCornersZ1()   { return fZ[1];      }
    virtual Double_t     GetCornersX2()   { return fX[2];      }
    virtual Double_t     GetCornersY2()   { return fY[2];      }
    virtual Double_t     GetCornersZ2()   { return fZ[2];      }
    virtual Double_t     GetCornersX3()   { return fX[3];      }
    virtual Double_t     GetCornersY3()   { return fY[3];      }
    virtual Double_t     GetCornersZ3()   { return fZ[3];      }

    virtual TPolyLine3D *GetPoly()       { return fPoly;      }

	virtual void        SetID(Int_t x)         { fID        = x;}
	virtual void        SetPlane(Int_t x)      { fPlane     = x; }
    virtual void        SetFamily(Int_t x)     { fFamily    = x; }
	virtual void        SetXcent(Double_t x)      { fXcent     = x; }
	virtual void        SetYcent(Double_t x)      { fYcent     = x; }
	virtual void        SetZcent(Double_t x)      { fZcent     = x; }
	virtual void        SetXcentP(Double_t x)     { fXcentP    = x; }
	virtual void        SetYcentP(Double_t x)     { fYcentP    = x; }
    virtual void        SetWidth(Double_t x)      { fWidth     = x; }
    virtual void        SetHeight(Double_t x)     { fHeight    = x; }
    virtual void        SetThickness(Double_t x)  { fThickness = x; }
	virtual void        SetActive(Bool_t x=kTRUE){ fActive    = x; }
    virtual void        SetCornersX0(Double_t x)  { fX[0]      = x; }
	virtual void        SetCornersY0(Double_t x)  { fY[0]      = x; }
	virtual void        SetCornersZ0(Double_t x)  { fZ[0]      = x; }
    virtual void        SetCornersX1(Double_t x)  { fX[1]      = x; }
	virtual void        SetCornersY1(Double_t x)  { fY[1]      = x; }
	virtual void        SetCornersZ1(Double_t x)  { fZ[1]      = x; }
    virtual void        SetCornersX2(Double_t x)  { fX[2]      = x; }
	virtual void        SetCornersY2(Double_t x)  { fY[2]      = x; }
	virtual void        SetCornersZ2(Double_t x)  { fZ[2]      = x; }
    virtual void        SetCornersX3(Double_t x)  { fX[3]      = x; }
	virtual void        SetCornersY3(Double_t x)  { fY[3]      = x; }
	virtual void        SetCornersZ3(Double_t x)  { fZ[3]      = x; }

	virtual void        SetPoly(TPolyLine3D * x) { fPoly      = x; }

    virtual void        Draw(Int_t Color = 3);
	virtual void        DrawPoint(Int_t Color = 3);
  
protected:
    Int_t      fID;            //  Pad ID       
    Int_t      fPlane;         //  Pad plane number
    Int_t      fFamily;        //  Pad Family 
    Double_t      fXcent;         //  Absolute X center position
    Double_t      fYcent;         //  Absolute Y center position
    Double_t      fZcent;         //  Absolute Z center position
	Double_t      fXcentP;        //  X center on plane
    Double_t      fYcentP;        //  Y center on plane
	Double_t      fX[4];          //  X corners of pad
	Double_t      fY[4];          //  Y corners of pad
    Double_t      fZ[4];          //  Z corners of pad
    Double_t      fWidth;         //  Pad width
    Double_t      fHeight;        //  Pad height
    Double_t      fThickness;     //  Si thickness
	Bool_t       fActive;        //  kFalse if dead;

	TPolyLine3D   * fPoly;       //  Outline of Pad
	TPolyMarker3D * fPolyM;      //  Center of Pad
    

public:

    ClassDef(TPhPad,1)  // PHOBOS detector pad class

};

#endif


