
///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhPhobDetector                                                //
//                                                                   //
//    Phob Detector base class                                       //
//                                                                   //                    //
//                                                                   //
//    Autheor : Christopher Conner (cmc@uic.edu)                     //
//    Created : 05/10/97                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// Phobos Classes
#include "TPhPhobDetector.h"

// Root Classes
#include <TObject.h>

// General Classes
#include <math.h>
#include <iostream.h>
#include <fstream.h>

//#define DEBUG

ClassImp(TPhPhobDetector)

// Description of the Phob Detector
// Contains most of the geometry specific stuff


void TPhPhobDetector::SetRowData   (Int_t *  x)
{
		if (fRow)
		{
#ifdef DEBUG
			cout << "Setting Row data ";
#endif
			for (Int_t i = 0; i < fNRows; i++)
			{
#ifdef DEBUG
				cout << x[i] << ':';
#endif
				fRow[i] = x[i];
			}
#ifdef DEBUG
			cout << endl;
#endif
		}
		else
		{
			cerr << "No place to store data" << endl;
		}

}

void TPhPhobDetector::SetColData   (Int_t *  x)
{
		if (fCol)
		{
#ifdef DEBUG
			cout << "Setting Col data ";
#endif
			for (Int_t i = 0; i < fNCols; i++)
			{
#ifdef DEBUG
				cout << x[i] << ':';
#endif
				fCol[i] = x[i];
			}
#ifdef DEBUG
			cout << endl;
#endif
		}
		else
		{
			cerr << "No place to store data" << endl;
		}

}
void TPhPhobDetector::SetupRowCol(Int_t nchan)
{
	if (fRow)
	{
		delete fRow;
	}
	if (fCol)
	{
		delete fCol;
	}
	fRow = new Int_t[nchan];
	fCol = new Int_t[nchan];
}

void TPhPhobDetector::FillGeometry(TPhPhobFamily * fam)
{
	for (Int_t i = 0; i< GetNChannels(); i++)
	{
    		TPhPad * pad = GetPad(i);
	   		pad->SetXcentP((fCol[i]+.5 -(fNCols/2.))*fPadWidth);
			pad->SetYcentP((fRow[i]+.5 -(fNRows/2.))*fPadHeight);
			pad->SetWidth(fPadWidth);
			pad->SetHeight(fPadHeight);
			pad->SetThickness(0.);
#ifdef DEBUG
			cout << "Row:" << fRow[i] << " Col:" << fCol[i] << " x:" << pad->GetXcentP() << " y:" << pad->GetYcentP() << endl;
#endif
	}

}

void TPhPhobDetector::CalcXYZ(TPhPhobFamily * fam, Int_t id)
{

  double DegRad = 45./atan(1.);
  double SinThx = sin(fThX/DegRad );
  double SinThy = sin(fThY/DegRad );
  double SinThz = sin(fThZ/DegRad );
  double CosThx = cos(fThX/DegRad );
  double CosThy = cos(fThY/DegRad );
  double CosThz = cos(fThZ/DegRad );

  double SinPhx = sin(fPhX/DegRad );
  double SinPhy = sin(fPhY/DegRad );
  double SinPhz = sin(fPhZ/DegRad );
  double CosPhx = cos(fPhX/DegRad );
  double CosPhy = cos(fPhY/DegRad );
  double CosPhz = cos(fPhZ/DegRad );

  Int_t start = 0;
  Int_t end   = GetNChannels();

  if (id > -1)
  {
	  if (id > GetNChannels())
	  {
		  cerr << "TPhDetector::CalcXYZ bad input ID" << endl;
		  return;
	  }
	  else
	  {
		  start = id;
		  end   = id;
	  }
  }

#ifdef DEBUG
  cout << "Calculating XYZ" << endl;
  cout << "Start " << start << " End " << end << endl;
#endif

  for (Int_t i = start; i < end; i++)
  { 

	TPhPad * pad = GetPad(i);
    Double_t x = pad->GetXcentP();
    Double_t y = pad->GetYcentP();
    Double_t z = 0;
#ifdef Double_t
	cout << x << ":" << y << ":" << z << endl;
#endif
    Double_t xx = x * SinThx* CosPhx + y * SinThy*CosPhy + z*SinThz*CosPhz + GetXcent();
    Double_t yy = x * SinThx* SinPhx + y * SinThy*SinPhy + z*SinThz*SinPhz + GetYcent();
    Double_t zz = x * CosThx         + y * CosThy        + z*CosThz        + GetZcent();
    pad->SetXcent(xx);
	pad->SetYcent(yy); 
	pad->SetZcent(zz); 
#ifdef Double_t
	cout << xx << ":" << yy << ":" << zz << endl;
#endif
	Double_t xa[4];
    Double_t ya[4];
	Double_t za[4] = {0.,0.,0.,0.};
	Double_t xxx[4];
	Double_t yyy[4];
	Double_t zzz[4];

	xa[0] = x - .5 * pad->GetWidth();
	xa[1] = x - .5 * pad->GetWidth();
	xa[2] = x + .5 * pad->GetWidth();
	xa[3] = x + .5 * pad->GetWidth();
	ya[0] = y - .5 * pad->GetHeight();
    ya[1] = y + .5 * pad->GetHeight();
	ya[2] = y + .5 * pad->GetHeight();
	ya[3] = y - .5 * pad->GetHeight();

    
	for (Int_t j = 0; j<4; j++)
	{
      xxx[j] = xa[j] * SinThx* CosPhx + ya[j] * SinThy*CosPhy + za[j]*SinThz*CosPhz + GetXcent();
      yyy[j] = xa[j] * SinThx* SinPhx + ya[j] * SinThy*SinPhy + za[j]*SinThz*SinPhz + GetYcent();
      zzz[j] = xa[j] * CosThx         + ya[j] * CosThy        + za[j]*CosThz        + GetZcent();  
	}

	pad->SetCornersX0(xxx[0]);
    pad->SetCornersY0(yyy[0]); 
	pad->SetCornersZ0(zzz[0]); 
	pad->SetCornersX1(xxx[1]);
    pad->SetCornersY1(yyy[1]); 
	pad->SetCornersZ1(zzz[1]); 
	pad->SetCornersX2(xxx[2]);
    pad->SetCornersY2(yyy[2]); 
	pad->SetCornersZ2(zzz[2]);
    pad->SetCornersX3(xxx[3]);
    pad->SetCornersY3(yyy[3]); 
	pad->SetCornersZ3(zzz[3]); 
  }
}

Bool_t TPhPhobDetector::Draw(Int_t color)
{
    TPhPad * pad;  

	ResetPadIterator();
    while(pad = GetNextPad())
    {
#ifdef DEBUG
		if (pad->GetID() == 1)
		{
	    cout << "Drawing Pad #" << pad->GetID() << endl; 
		}
#endif
		pad->Draw(color);
    }
    return kTRUE;
}
