#ifndef _PhPhobDetector_INCLUDED_
#define _PhPhobDetector_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhPhobDetector                                                //
//                                                                   //
//    Phob DetDetector base class                                    //
//                                                                   //                    //
//                                                                   //
//    Autheor : Christopher Conner (cmc@uic.edu)                     //
//    Created : 05/11/97                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// Phobos Classes

#include "TPhDetector.h"
#include "TPhPhobFamily.h"

class TPhPhobDetector : public TPhDetector
{

public:
    TPhPhobDetector()    {fDetTypeID = TPhPhobFamily::kFamID; fRow = NULL; fCol = NULL;}
    ~TPhPhobDetector() {if (fRow) {delete fRow;} if (fCol){delete fCol;}}
	
	Int_t    GetNRows()       {return fNRows;}
	Int_t    GetNCols()       {return fNCols;}
	Double_t GetPadWidth()    {return fPadWidth;}
	Double_t GetPadHeight()   {return fPadHeight;}
	Int_t *  GetRowData()     {return fRow;}
    Int_t *  GetColData()     {return fCol;}
    Int_t    GetRow(Int_t ID) {return fRow[ID];}
	Int_t    GetCol(Int_t ID) {return fCol[ID];}

	void SetNRows     (Int_t x)            {fNRows = x;}
    void SetNCols     (Int_t x)            {fNCols = x;}
	void SetRow       (Int_t ID, Int_t x)  {fRow[ID] = x;}
	void SetCol       (Int_t ID, Int_t x)  {fCol[ID] = x;}
    void SetPadWidth  (Double_t padWidth)  {fPadWidth = padWidth;}
    void SetPadHeight (Double_t padheight) {fPadHeight = padheight;}
	void SetupRowCol  (Int_t nchan);
	void SetRowData   (Int_t * x);
	void SetColData   (Int_t * x);

	void     FillGeometry(TPhPhobFamily * fam);
    void     CalcXYZ(TPhPhobFamily * fam, Int_t id = -1);
	Bool_t Draw(Int_t color=-1);

protected:

	Int_t    fNRows;      // Number of rows
	Int_t    fNCols;      // Number of columns
	Int_t *  fRow;        // Row position of readout
	Int_t *  fCol;        // Col position of readout
	Double_t fPadWidth;   // Width of channels
	Double_t fPadHeight;  // Height of channels

    ClassDef(TPhPhobDetector,1) // PHOBOS geometry and calibration interface
};

#endif

