
///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhPhobFamily                                                  //
//                                                                   //
//    Phob DetFamily base class                                      //
///////////////////////////////////////////////////////////////////////

// Phob Classes
#include "TPhPhobFamily.h"
#include "TPhPhobDetector.h"

// Root Classes
#include <TObject.h>

// General Classes
#include <math.h>
#include <iostream.h>
#include <fstream.h>

//#define DEBUG

ClassImp(TPhPhobFamily)

// Description of the Phob Detector family
// Contains most of the geometry specific stuff

// Initialize TPhPhobFamily constants
const Int_t  TPhPhobFamily::kFamID(5);


Bool_t   TPhPhobFamily::ReadGeometry(char *inputname1, char * inputname2)
{ 
    char*   defaultfile1 = "Phobgeom.dat";
    char*   defaultfile2 = "PhobTranslat.dat" ;
    char* filename1;
    char* filename2;


    if(!inputname1)
     {
     cout << "No Phobos Pad Description File! Trying:" << defaultfile1 << endl;
     filename1 = defaultfile1;
     }
    else
     {
     filename1 = inputname1;
     }

    
    ifstream infile1(filename1, ios::in | ios::nocreate);
	if (!infile1)
	{
		cerr << "Can not open '" << filename1 << "' for input." << endl;
		return kFALSE;
	}
 
    if(!inputname2)
    {
		cout << "No Phobos mapping description File! Trying:" << defaultfile2 << endl;
        filename2 = defaultfile2;
	}
	else
	{
		filename2 = inputname2;
    }

    
    ifstream infile2(filename2, ios::in | ios::nocreate);
	if (!infile2)
	{
		cerr << "Can not open '" << filename2 << "' for input." << endl;
		return kFALSE;
	}

        Int_t detID;
		Int_t nchan;
		Int_t nrows;
		Int_t ncols;
		Double_t padwidth;
		Double_t padheight;
		TPhPhobDetector * det;

#ifdef DEBUG
		cout << "Reading Phob Descritption file" << endl;
#endif
        while ((infile1 >> detID >> nchan >> nrows >> ncols >> padwidth >> padheight))
		{
			det = (TPhPhobDetector *)(GetDetector(detID));
			if (!det)
			{
				det = new TPhPhobDetector();
				det->SetupAllData(nchan,0);
				det->SetupRowCol(nchan);
				det->SetDetID(detID);
				det->SetDetTypeID(kFamID);
				det->SetNRows(nrows);
				det->SetNCols(ncols);
				det->SetPadWidth(padwidth);
				det->SetPadHeight(padheight);
				
				AddDetector(det);
			}
			for (Int_t i=0; i < nchan; i++)
			{
				TPhPad * pad = new TPhPad();
				pad->SetPlane(detID);
                det->AddPad(pad, i);
			}
    }
    infile1.close();

#ifdef DEBUG
		cout << "Done Reading Phob Descritption file" << endl;
#endif


	// setup lookup array

	Int_t ichan;
	Int_t ndet = GetNDetectors();
	Int_t row;
	Int_t col;
	for (Int_t idet = 1; idet <= ndet; idet++)
	{
		det = (TPhPhobDetector *) GetDetector(idet);
		Int_t nchan = det->GetNChannels();
		for (Int_t counter = 1; counter<=nchan; counter++)
		{
			if ((infile2 >> ichan
				         >> col
		     			 >> row))
			{
				if(counter != ichan)
				{
					cerr << "Error in Phobos pad description file." << endl;
					return kFALSE;
				}
#ifdef DEBUG
				cout << "Row:" << row << " Col:" << col << endl;
#endif
				det->SetRow(counter - 1,row);
				det->SetCol(counter - 1,col);

			}
			else
			{
				cerr << "Not enough lines in Phobos map description file." << endl;
			}
		}
	}
#ifdef DEBUG
		cout << "Done Creating Pad Mapping " << endl;
#endif

	// Fill geometry info

	ResetDetectorIterator();
	det = (TPhPhobDetector *) (GetNextDetector());
    while (det)
	{
	 det->FillGeometry(this);
	 det->CalcXYZ(this, -1);
#ifdef DEBUG
		cout << "Filling Geometry Detector#" << det->GetDetID() << endl;
		TPhPad * pad = det->GetPad(1);
		cout << "Point XYZ ";
		cout << pad->GetCornersX1() << ':';
		cout << pad->GetCornersY1() << ':';
		cout << pad->GetCornersZ1() << ':';
#endif

	 det = (TPhPhobDetector *) (GetNextDetector());
	}

    return kTRUE;
}


