
#include "TPhSetup.h"
#include "TPhTeleFamily.h"
#include "TPhTeleDetector.h"
#include "TPhWA98Fam.h"
#include "TPhPhobFamily.h"
#include "TPhEvent.h"

#include <iostream.h>
#include <fstream.h>

#define DEBUG
//
ClassImp(TPhSetup)

/////////////////////////////////////////////////////////////
//
//  TPhSetup sets up the geometry structure for the 
//  TestBeam setup
//
//////////////////////////////////////////////////////////////


void TPhSetup::SetupGeometry(TPhEvent * event, char * inputname1)
{
    // Create an event and make the framework
	// as chosen by input file
    // default file is 
	// filename1 = ./AllGeometry.dat

	char*   defaultfile1 = "AllGeometry.dat";
	ifstream infile1;


    if(!inputname1)
    {
		cout << "No Geometry File! Trying:" << defaultfile1 << endl;
		infile1.open(defaultfile1, ios::in | ios::nocreate);
		if (!(infile1))
		{
			cerr << "Can not open '" << defaultfile1 << "' for input." << endl;
			return;
		}
	}
	else
	{
		infile1.open(inputname1, ios::in | ios::nocreate);
		if (!(infile1))
		{
			cerr << "Can not open '" << inputname1 << "' for input." << endl;
			return;
		}
		
    }
    
#ifdef DEBUG
	cout << "File openend sucessfully" << endl;
#endif
 
        char name[4];
		Int_t typeID;
		Int_t  detID;
		Double_t xcent;
		Double_t ycent;
		Double_t zcent;
		Double_t thx;
		Double_t phx;
		Double_t thy;
		Double_t phy;
		Double_t thz;
		Double_t phz;
		
		TPhDetector * det;
		TPhDetFamily * fam;

#ifdef DEBUG
	cout << "About to ReadFile" << endl;
	Int_t count = 0;
#endif

        while (infile1.good())
		{
			infile1 >> name;
	infile1 >> typeID;
    infile1 >>  detID;
	infile1 >>  xcent;
	infile1 >>  ycent;
	infile1 >>  zcent;
	infile1 >>  thx;
	infile1 >>  phx;
	infile1 >>  thy;
	infile1 >>  phy;
	infile1 >>  thz;
	infile1 >>  phz;
		
#ifdef DEBUG
			cout << "analysing line # " << ++count << endl;
#endif
			fam = event->GetFamily(typeID);
			if (!fam)
			{
				switch(typeID) 
				{
					// constant not working
					// temporaily hard code
//				case TPhTeleFamily::kFamID:
				case 7:
#ifdef DEBUG
					cout << "Creating the Telescope Family" << endl;
#endif
					fam = new TPhTeleFamily();
					event->AddFamily(fam);
					break;
//				case TPhWA98Family::kFamID:
				case 12:
#ifdef DEBUG
					cout << "Creating the WA98 Family" << endl;
#endif

					fam= new TPhWA98Family();
					event->AddFamily(fam);
					break;
				default:
					cerr << "Invalid Detector Family ID number" << endl;
					cerr << "FamID " << typeID << " is undefined." << endl;
					return;
					break;
				}
			}
			det = fam->GetDetector(detID);
			if (!det)
			{
#ifdef DEBUG2
				cout << "Returned zero pointer" << endl;
#endif
				switch(typeID) 
				{
//			case TPhTeleFamily::kFamID:
				case 7:
					det = new TPhTeleDetector();
					break;
//			case TPhWA98Family::kFamID:
				case 12:
					det = new TPhDetector();
					break;
				default:
					cerr << "Error in code." << endl;
					cerr << "FamID" << typeID << " is undefined." << endl;
					return;
					break;
				}
				
			det->SetDetID(detID);
			fam->AddDetector(det);
			}
			det->SetDetTypeID(typeID);
			det->SetXcent(xcent);
			det->SetYcent(ycent);
			det->SetZcent(zcent);
			det->SetThX(thx);
			det->SetPhX(phx);
			det->SetThY(thy);
			det->SetPhY(phy);
			det->SetThZ(thz);
			det->SetPhZ(phz);
		}
		
    infile1.close();

#ifdef DEBUG
	 cout << "Done Setting UP" << endl;
	 cout << "Families Created:" << event->GetNFamily() << endl;
	 cout << "Detectors Created:" 
		 << event->GetNFamilyMembers(TPhTeleFamily::kFamID) +
		 event->GetNFamilyMembers(TPhWA98Family::kFamID) << endl;
#endif


	return;
}
