

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhTeleFamily                                                  //
//                                                                   //
//    Tele DetFamily base class                                      //
//                                                                   //                    //
//                                                                   //
//    Autheor : Christopher Conner (cmc@uic.edu)                     //
//    Created : 05/10/97                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
///////////////////////////////////////////////////////////////////////

//Phobos Classes
#include "TPhTeleFamily.h"
#include "TPhTeleDetector.h"

// Root Classes
#include <TObject.h>

// General Classes
#include <math.h>
#include <iostream.h>
#include <fstream.h>

//#define DEBUG

ClassImp(TPhTeleFamily)

// Description of the Tele Detector family
// Contains most of the geometry specific stuff

// Initialize TPhTeleFamily constants
const Int_t  TPhTeleFamily::kFamID(7);
const Int_t	 TPhTeleFamily::kChanVA1(128);


Bool_t   TPhTeleFamily::ReadGeometry(Char_t *inputname1, Char_t * inputname2)
{
    // read geometry file and create and fill the tree
    // of family members and pads
    // default files are 
	// filename1 = ./Telegeom.dat (datastream discription) 
	Char_t*   defaultfile1 = "Telegeom.dat";
    Char_t* filename1;


    if(!inputname1)
    {
		cout << "No Tele Pad Description File! Trying:" << defaultfile1 << endl;
        filename1 = defaultfile1;
	}
	else
	{
		filename1 = inputname1;
    }

#ifdef DEBUG
	cout << "About to open file" << endl;
#endif
    
    ifstream infile1(filename1, ios::in | ios::nocreate);
	if (!infile1)
	{
		cerr << "Can not open '" << filename1 << "' for input." << endl;
		return kFALSE;
	}

 
        Int_t detID;
		Int_t nchan;
		Double_t padwidth;
		TPhTeleDetector * det;

#ifdef DEBUG
	cout << "About to read file" << endl;
#endif
 
        while ((infile1 >> detID >> nchan >> padwidth))
		{
			det = (TPhTeleDetector *) (GetDetector(detID));
			if (!det)
			{
				det = new TPhTeleDetector();
				det->SetupAllData(nchan,0);
				det->SetDetID(detID);
				det->SetDetTypeID(kFamID);
				AddDetector(det);
				
				det->SetPadWidth(padwidth);
			}
			for (Int_t i=0; i < nchan; i++)
			{
				TPhPad * pad = new TPhPad();
				pad->SetPlane(detID);
                det->AddPad(pad, i);
			}
    }
    infile1.close();

	ResetDetectorIterator();
	det = (TPhTeleDetector *) (GetNextDetector());
    while (det)
	{

	 det->FillGeometry();
	 det->CalcXYZ();
#ifdef DEBUG
		cout << "Filling Geometry Detector#" << det->GetDetID() << endl;
		TPhPad * pad = det->GetPad(1);
		cout << "Point XYZ ";
		cout << pad->GetCornersX1() << ':';
		cout << pad->GetCornersY1() << ':';
		cout << pad->GetCornersZ1() << ':';
#endif

	 det = (TPhTeleDetector *) (GetNextDetector());
	}

    return kTRUE;
}