
#include "TPhTestData.h"

#include <math.h>
#include <stdlib.h>

#include "TPhTeleFamily.h"
#include "TPhPhobFamily.h"
#include "TPhWA98Fam.h"
#include "TPhDetHit.h"
#include "TPhPhobDetector.h"

//#define DEBUG
//#define DEBUG2
//#define WA98
//#define FILE
//#define COUNT
#define INTCHECK

ClassImp(TPhTestData)

////////////////////////////////////////////////////////////////
//
// TPhTestData provides an interface to output from the
// testbeam DAQ.
//
////////////////////////////////////////////////////////////////

// Initialize TPhTestData constants
const Int_t TPhTestData::kKEY = Int_t(1997);
const Int_t TPhTestData::kHEADSIZE = Int_t(80);


TPhTestData::TPhTestData()
{
// Constructor. 
	cout << "Consructing TPhTestData (no filename)" << endl;

    fEventNumber = 0;
	fEndofFile   = 0;
	fBeginEvent  = 0;
	fEOF = kFALSE;
	
	if (sizeof(Int_t) < 4 )
	{
		cerr << "This program will not work on this machine" << endl;
		cerr << "Need to change the way GetInt routine works" << endl;
		cerr << "in Class TPhTestData." << endl;
	}
}

TPhTestData::TPhTestData(char * filename)
{
	fEventNumber = 0;
	fEndofFile   = 0;
	fBeginEvent  = 0;
	fEOF = kFALSE;

// Constructor. Set fFile and fEventNumber to NULL
// and open the file pointed to by *filename.
	cout << "Constructing TPhTestData with name" << filename << endl;

    if (!Open(filename))
	{
		cout << "Open failed" << endl;
	}

	if (sizeof(Int_t) < 4 )
	{
		cerr << "This program will not work on this machine" << endl;
		cerr << "Need to change the way GetInt routine works" << endl;
		cerr << "in Class TPhTestData." << endl;
	}
#ifdef FILE
	streampos mark = fFile.tellg();
	cout << "File begins at " << mark << endl;
#endif
}

Bool_t TPhTestData::Open(char * filename)
{
// Open the Test data file *fname. Returns kFALSE
// if the file can't be opened.
// Otherwise the fFileName data member is set to the 
// corresponding File

    if(!filename)
    {
        cerr << "No filename given!" << endl;
        return kFALSE;
    }

	fFile.open(filename,ios::in | ios::binary | ios::nocreate);
	if (!fFile)
	{
		cerr << "Can not open '" << filename << "' for input." << endl;
		return kFALSE;
	}
    else
    {
#ifdef DEBUG 
		cout << "File opened sucessfully." << endl;
#endif
	
	fFile.seekg(0,ios::end);
	fEndofFile = fFile.tellg();
	fFile.seekg(0,ios::beg);
	fEOF = kFALSE;
#ifdef DEBUG
	cout << "File is " << fEndofFile << " bytes long." << endl;
#endif
		return kTRUE;
    }
	
}

Bool_t TPhTestData::Close()
{
// Close fFile. Returns kFALSE if no file was open.

    if(fFile)
    {
        fFile.close();
        return kTRUE;
    }
    else
    {
        return kFALSE;
    }
}

Bool_t TPhTestData::GetNextEvent(TPhEvent * event)
{ 


// Reads one event from the Testbeam 1997 data file.
// Overwrites all the hits in the event.
	if (!fFile.good())
	{
		cerr << "File is not open." << endl;
		return kFALSE;
	}

	fBeginEvent = fFile.tellg();
#ifdef FILE
	cout << "Begining of Event is " << fBeginEvent <<endl;
#endif

	if (event)
	{
#ifdef DEBUG
        cout << "Clear hits from Event." << endl;
#endif
#ifdef COUNT
		cout << "Number of hits before Drop = " << TPhDetHit::GetNCreated() << endl;
#endif
        event->DropAllHits();

#ifdef COUNT
		cout << "Number of hits after Drop = " << TPhDetHit::GetNCreated() << endl;
#endif

		
#ifdef DEBUG
        cout << "Done Clearing hits from Event." << endl;
#endif
	}
	else
	{
		cerr << "Event Stucture not setup" << endl;
		exit (-1);
	}

	if (!ReadHeader())
	{
		cerr << "Error in EventHeader" << endl;
		return kFALSE;
	}

	event->SetSoftware(fSoftware   );
    event->SetEvent   (fEventNumber);
    event->SetRun     (fRunNumber  );
    event->SetDate    (fDate       );
    event->SetTime    (fTime       );
    event->SetDataType(TPhEvent_t (fDataType));

	if (event->GetDataType() == kERROR)
	{
		fFile.seekg(fNextEvent);
		return kTRUE;
	}

#ifdef DEBUG
	cout << "Reading Phobos detector" << endl;
#endif

	if (!(ReadPhobos(event)))
	{
		cerr << "Error reading Phobos Detector" << endl;
	}

	// read out TDC
	if (fNumberTDC > 0)
	{
		for (Int_t i = 0; i<fNumberTDC; i++)
		{
			Int_t dummy = GetInt(2);
		}
	}

#ifdef DEBUG
	cout << "Reading Telescope" << endl;
#endif

	if(!(ReadTelescope(event)))
	{
		cerr << "Error reading Taiwan Telescope" << endl;
	}


#ifdef DEBUG
	cout << "Reading WA98" << endl;
#endif

	if (!(ReadWA98(event)))
	{
		cerr << "Error reading WA98 detector" << endl;
	}

#ifdef DEBUG
	cout << "Event Read" << endl;
#endif

	streampos mark = fFile.tellg();
	if (mark != fNextEvent)
	{
		cerr << "File not read correctly " << fNextEvent << " != " << mark <<endl; 
		cerr << "Event number is : " << fEventNumber << endl;
		exit (-1);
	}


    return kFALSE;
}

Bool_t TPhTestData::ReadHeader(Int_t show)
{

	Int_t fKey1;
	Int_t fKey2;

#ifdef FILE
	streampos mark1 = fFile.tellg();
	cout << "Position before header read is " << mark1 << endl;
#endif

	fSoftware    = GetInt();
	fRunNumber   = GetInt();
	fEventNumber = GetInt();
	fDate        = GetInt();
	fTime        = GetInt();
	fDataType    = GetInt();
	fKey1        = GetInt();
	fNumberBytes = GetInt();
	fNumberVA1   = GetInt();
	fVA1gain     = GetInt();
	fBytesADC    = GetInt();
	fNumberADC   = GetInt();
	fBytesTDC    = GetInt();
	fNumberTDC   = GetInt();
	fBytesWA98   = GetInt();
	fNumberFADC  = GetInt();
	fWA98MAR     = GetInt();
	fWA98CSR     = GetInt();
	fNumberMIP   = GetInt();
    fKey2        = GetInt();

#ifdef FILE
	mark1 = fFile.tellg();
	cout << "Position after header read is " << mark1 << endl;
#endif

#ifdef DEBUG
	show = 1;
#endif

	if (show > 0) 
	{
	cout << "Software  :" << fSoftware    << endl;
	cout << "Run       :" << fRunNumber   << endl;
	cout << "Event     :" << fEventNumber << endl;
	cout << "Date      :" << fDate        << endl;
	cout << "Time      :" << fTime        << endl;
	cout << "DataType  :" << fDataType    << endl;
	cout << "Key       :" << fKey1        << endl;
	cout << "N Bytes   :" << fNumberBytes << endl;
	cout << "N VA1     :" << fNumberVA1   << endl;
	cout << "VA1 Gain  :" << fVA1gain     << endl;
	cout << "Bytes ADC :" << fBytesADC    << endl;
	cout << "N ADC     :" << fNumberADC   << endl;
	cout << "Bytes TDC :" << fBytesTDC    << endl;
	cout << "N TDC     :" << fNumberTDC   << endl;
	cout << "Bytes WA98:" << fBytesWA98   << endl;
	cout << "N FADC    :" << fNumberFADC  << endl;
	cout << "WA98 MAR  :" << fWA98MAR     << endl;
	cout << "WA98 CSR  :" << fWA98CSR     << endl;
	cout << "N MIP     :" << fNumberMIP   << endl;
    cout << "Key       :" << fKey2        << endl;
	}

	if (!fFile.good())
	{
		cerr << "File error during read header" << endl;
		if(fFile.eof())
		{
			cout << "End of file reached" << endl;
			fEOF = kTRUE;
		}
		return kFALSE;
	}

    streampos mark = fFile.tellg();
	fNextEvent = mark + fNumberBytes - kHEADSIZE;

	if (fKey2 != kKEY || fKey1 != kKEY)
	{
		cerr << "File header in wrong format." << endl;
		
			cerr << "Event size:" << fNumberBytes << endl;
			cerr << "VA1 bytes :" << fNumberVA1*TPhTeleFamily::kChanVA1 << endl;
			cerr << "ADC bytes :" << fNumberADC << " x 2 = " << fBytesADC << endl;
			cerr << "TDC bytes :" << fNumberTDC << " x 2 = " << fBytesTDC << endl;
			cerr << "WA98 bytes:" << fBytesWA98 << endl;
			cerr << "Key = " << kKEY << " read " << fKey1 << ':' << fKey2 << endl;
		    cerr << "Current position = " << mark << endl;
			cerr << "End of File      = " << fEndofFile << endl;
			cerr << "Next Event       = " << fNextEvent << endl;
			exit (-1);
		return kFALSE;
	}

	if (fNextEvent > fEndofFile)
	{
	    	cerr << " Last Event (not written fully)" << endl;
		    cerr << "Current position = " << mark << endl;
			cerr << "End of File      = " << fEndofFile << endl;
			cerr << "Next Event       = " << fNextEvent << endl;
			if (fFile.eof()) 
				cout << "End of File reached" << endl;
		fEOF = kTRUE;
	}
	else if (fNextEvent == fEndofFile)
	{
		cout << "Last Event" << endl;
		fEOF = kTRUE;
	}
	else
	{
		fFile.seekg(fNumberBytes-4,ios::cur);
		Int_t key = GetInt();
		fFile.seekg(mark);
		if (key != kKEY)
		{
			cerr << "Event size does not match header description" << endl;
			cerr << "Event size:" << fNumberBytes << endl;
			cerr << "VA1 bytes :" << fNumberVA1*TPhTeleFamily::kChanVA1 << endl;
			cerr << "ADC bytes :" << fNumberADC << " x 2 = " << fBytesADC << endl;
			cerr << "TDC bytes :" << fNumberTDC << " x 2 = " << fBytesTDC << endl;
			cerr << "WA98 bytes:" << fBytesWA98 << endl;
			cerr << "Key = " << kKEY << " read " << fKey1 << ':' << fKey2 << endl;
		    cerr << "Current position = " << mark << endl;
			cerr << "End of File      = " << fEndofFile << endl;
			cerr << "Next Event       = " << fNextEvent << endl;
			cerr << "Next Key         = " << key << endl;
			exit (-1);
		}

	}

	return kTRUE;
}

	
Int_t TPhTestData::GetInt(Int_t size)
{
	// Read an integer from a binary file
	// Max size allowed is 4
	// Default size is 4

	char ch;
	Int_t i;
    Int_t ans;

	ans = 0;
	for (int loop = 0; loop < size; loop++)
	{
		ans *=256;
		fFile.get(ch);
		i = Int_t(ch);
		if (i < 0)
		{
			i += 256;
		}
		ans += i;

	}
	return ans;
	
    
}

Bool_t TPhTestData::ReadPhobos(TPhEvent * event)
{
	TPhDetFamily   * fam;
	TPhDetector * det;
	Int_t extra = 0;

	// Read out Phobos Detector
	if (fNumberADC > 0)
	{
		
		fam = event->GetFamily(TPhPhobFamily::kFamID);
		if (!fam)
		{
			 cerr << "Phobos Family not setup" << endl;
		}
		Int_t ndetect = fam->GetNDetectors();
		Int_t nchan = 0;

		 fam->ResetDetectorIterator();
		 det = fam->GetNextDetector();
		while (det)
		{
			nchan += det->GetNChannels();
			det = fam->GetNextDetector();
		}

		if (nchan > fNumberADC)
		{
			cerr << "Not enough ADC to fill Phobos Detector. Geometry:" << nchan << " Header:" << fNumberADC << endl;
			for (Int_t i = 0; i<fNumberADC; i++)
			{
				 Int_t dummy = GetInt(2);
			}

		}
		else
		{			
			if (nchan < fNumberADC)
			{
				extra = fNumberADC - nchan;
				// Temporary fix (for extra ADC not in geometry)
				// Make second phobos detector with rest of ADC.
				cout << "Creating storage for extra ADC" << endl;
				TPhPhobDetector * det1 = new TPhPhobDetector();
				det1->SetupAllData(extra,0);
				det1->SetDetID(2);
				det1->SetDetTypeID(TPhPhobFamily::kFamID);
				det1->SetNRows(1);
				det1->SetNCols(extra);
				det1->SetPadWidth(1.);
				det1->SetPadHeight(1.);
				
				fam->AddDetector(det1);
				
			    for (Int_t i=0; i < extra; i++)
			    {
				    TPhPad * pad = new TPhPad();
				    pad->SetPlane(2);
                    det1->AddPad(pad, i);
			    }
				ndetect++;
			}
			for (Int_t i = 1; i< ndetect+1; i++)
			{
				det = fam->GetDetector(i);
				if (!det)
				{
					 cerr << "Phobos Detector not setup" << endl;
				}
				
				Int_t size = det->GetNChannels();
				Short_t * array = new Short_t[size];
				for (Int_t j = 0; j< size; j++)
				{
					 array[j] = GetInt(2);
				}
				if (det)
				{
#ifdef DEBUG2
					cout << "Setting raw data" << endl;
					cout << "DataIN";
					for (Int_t j = 0; j < 11 ; j++)
					{
						cout << ':' << array[j];
					}
					cout << endl;
#endif
		
	 				 det->SetRawData(array);
					 delete array;
					 array = NULL;
					 det->SetEventPointer(event);
					 
#ifdef DEBUG2
					cout << "DataSet" << endl;
					cout << "DataOut";
					for (j = 0; j < 11 ; j++)
					{
						cout << ':' << det->GetRaw(j);
					}
					cout << endl;
#endif

				}
			}
		}
	}
	return kTRUE;
}


Bool_t TPhTestData::ReadTelescope(TPhEvent * event)
{
	TPhDetFamily * fam;
	TPhDetector  * det;

	// Read out Telescope
	if (fNumberVA1 > 0)
	{
		fam = event->GetFamily(TPhTeleFamily::kFamID);
		if (!fam)
		{
			 cerr << "Telescope Family not setup" << endl;
		}
		Int_t ndetect = fam->GetNDetectors();
		Int_t nchan = 0;

		 fam->ResetDetectorIterator();
		 det = fam->GetNextDetector();
		while (det)
		{
			nchan += det->GetNChannels();
			det = fam->GetNextDetector();
		}

		if (nchan != fNumberVA1*TPhTeleFamily::kChanVA1)
		{
			 cerr << " Number of VA1 does not match geometry" << endl;
			 cerr << "Header:" << fNumberVA1*TPhTeleFamily::kChanVA1 << " Geometry:" << nchan << endl;
			 for (Int_t i = 0; i<fNumberVA1*TPhTeleFamily::kChanVA1; i++)
			 {
				 Int_t dummy = GetInt(1);
			}

		}
		else
		{
			for (Int_t i = 1; i< ndetect+1; i++)
			{
				
				det = fam->GetDetector(i);
				if (!det)
				{
					 cerr << "Telescope not setup" << endl;
				}
				Int_t size = det->GetNChannels();
				Short_t * array = new Short_t[size];

				for (Int_t j = 0; j< size; j++)
				{
					 array[j] = GetInt(1);
				}
				if (det)
				{
#ifdef DEBUG2
					cout << "Setting raw data" << endl;
					cout << "DataIN";
					for (Int_t j = 0; j < 11 ; j++)
					{
						cout << ':' << array[j];
					}
					cout << endl;
#endif
		
	 				 det->SetRawData(array);
					 delete array;
					 array = NULL;
					 det->SetEventPointer(event);
					 
#ifdef DEBUG2
					cout << "DataSet" << endl;
					cout << "DataOut";
					for (j = 0; j < 11 ; j++)
					{
						cout << ':' << det->GetRaw(j);
					}
					cout << endl;
#endif

				}
			}
		}
	}
	return kTRUE;
}

Bool_t TPhTestData::ReadWA98(TPhEvent * event)
{
	TPhDetFamily * fam;
	TPhDetector  * det;

	// Read out WA98 detector
	if (fBytesWA98 > 0)
	{
		fam = event->GetFamily(TPhWA98Family::kFamID);
#ifdef WA98
		cout << "Reading WA98" << endl;
		if (fam)
		{
			cout << "Family exists" << endl;
		}
		else
		{
			cout << "Family is missing" << endl;
		}
#endif
		if (!fam)
		{
			 cerr << "WA98 Family not setup" << endl;
		}
		Int_t ndetect = fam->GetNDetectors();
		Int_t nchan = 0;
#ifdef WA98
		cout << "Number of detectors is " << ndetect;
#endif
		 fam->ResetDetectorIterator();
		 det = fam->GetNextDetector();
		while (det)
		{
			nchan += det->GetNChannels();
			det = fam->GetNextDetector();
		}
#ifdef WA98
		cout << "Number of channels is " << nchan << endl;
#endif
		if (nchan != fBytesWA98)
		{
			 cerr << " Number of WA98 bytes does not match geometry" << endl;
			 cerr << " Geometry = " << nchan << " data = " << fBytesWA98 << endl;
			 for (Int_t i = 0; i<fBytesWA98; i++)
			 {
				 Int_t dummy = GetInt(1);
			}

		}
		else
		
		{
			for (Int_t i = 1; i< ndetect+1; i++)
			{
				
				det = fam->GetDetector(i);
				if (!det)
				{
					 cerr << "WA98 not setup" << endl;
				}
				Int_t size = det->GetNChannels();
				Short_t * array = new Short_t[size];
#ifdef WA98
		cout << "About to read data " << endl;
#endif

				for (Int_t j = 0; j< size; j++)
				{
					 array[j] = GetInt(1);
#ifdef WA98
					 cout << array[j] << ':';
#endif
				}
#ifdef WA98
				cout << endl;
#endif

				if (det)
				{
#ifdef DEBUG2
					cout << "Setting raw data" << endl;
					cout << "DataIN";
					for (Int_t j = 0; j < 11 ; j++)
					{
						cout << ':' << array[j];
					}
					cout << endl;
#endif
		
	 				 det->SetRawData(array);
					 delete array;
					 array = NULL;
					 det->SetEventPointer(event);
					 
#ifdef DEBUG2
					cout << "DataSet" << endl;
					cout << "DataOut";
					for (j = 0; j < 11 ; j++)
					{
						cout << ':' << det->GetRaw(j);
					}
					cout << endl;
#endif

				}
			}
		}
	}
	else
	{
#ifdef WA98
		cout << "No WA98 Data to read" << endl;
#endif

	}
	return kTRUE;
}

void TPhTestData::ShowHeader()
{
	// This routine will read the header and show the values
	// It will return to whatever file position it was in before

	streampos mark = fFile.tellg();
	cout << "Current FilePosition is:" << mark << endl;
	fFile.seekg(fBeginEvent);
	cout << "Start of event is :" << fBeginEvent << endl;

	if (!ReadHeader(1))
	{
		cerr << endl;
	}
	fFile.seekg(mark);
	cout << "Returned file to  :" << mark << endl;

	return;
}
