#ifndef _PhTestData_INCLUDED_
#define _PhTestData_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhTestData                                                    //
//                                                                   //
//    PHOBOS testbeam 1997 DAQ data format                           //
//                                                                   //
//    TPhTestData handles raw data in Testbeam 1997 format           //
//                                                                   //
//    Author  : Christopher Conner (cmc@uic.edu)                     //
//    Created : 05/11/97                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// Root Classes
#include <TObject.h>

// General Classes
#include <iostream.h>
#include <fstream.h>

#include "TPhEvent.h"

class TPhTestData : public TObject {

public:
    TPhTestData(); 
    TPhTestData(char *filename);

    virtual ~TPhTestData() { fFile.close(); }


    virtual Bool_t    Open(char *fname);
    virtual Bool_t    Close();
    virtual Bool_t    GetNextEvent(TPhEvent * event);
	virtual void      ShowHeader();
	virtual Bool_t	  IsEOF()  { return fEOF; }
	virtual Bool_t    IsOpen()   { return fFile.good();}

private:

	Bool_t   ReadHeader(Int_t show = -1);
	Bool_t   ReadTelescope(TPhEvent * event);
	Bool_t   ReadPhobos(TPhEvent * event);
	Bool_t   ReadWA98(TPhEvent * event);

	Int_t    GetInt(Int_t size=4);

	Int_t    fSoftware;              // Software version number
	Int_t    fRunNumber;             // Run number 
	Int_t    fEventNumber;           // event number
	Int_t    fDate;                  // date (YYYYMMDD)
	Int_t    fTime;                  // time (HHMMSS)
	Int_t    fDataType;              // data type (signal = 1, pedestal = 2, calibration = 3, error = 4)
	Int_t    fNumberBytes;           // event size in bytes (includes header)
	Int_t    fNumberVA1;             // number of VA1 (128 channels 1 byte each)
	Int_t    fVA1gain;               // gain of ADC for VA1
	Int_t    fBytesADC;              // number of ADC bytes (2 bytes per chan)
	Int_t    fNumberADC;             // number of ADC channels (12 chan per module)
	Int_t    fBytesTDC;              // number of TCD bytes (2 bytes per chan)
	Int_t    fNumberTDC;             // number of TDC channels (12 chan per module)
	Int_t    fBytesWA98;             // number of WA98 bytes (1 byte per chan)
	Int_t    fNumberFADC;            // number of FADC samples (4 bytes per sample)
	Int_t    fWA98MAR;               // MAR for WA98
	Int_t    fWA98CSR;               // CSR for WA98
    Int_t    fNumberMIP;             // Number of MIP's

	//  data format =  header ( 20X4 bytes ) + ADC(x2bytes) + TDC(x2bytes) +  VA1(29X128 bytes )  + WA98 (4096bytes)      
    
	char      fFileName[256];        // File name for Test file containing raw hits
    ifstream  fFile;                 // Input file
    streampos fEndofFile;            // Size of file
	streampos fBeginEvent;           // Location of begining of event
	streampos fNextEvent;            // Location of next event
	Bool_t    fEOF;                  // End of File Reached

	static const Int_t kKEY;         // 1997 Test key
	static const Int_t kHEADSIZE;    // Size of header

public:
    ClassDef(TPhTestData,1) // Interface to Testbeam 1997 DAQ
};

#endif



