#ifndef _PhWA98Pad_INCLUDED_
#define _PhWA98Pad_INCLUDED_

///////////////////////////////////////////////////////////////////////
//                                                                   //
//    TPhWA98Pad                                                     //
//                                                                   //
//    WA98 Detector pad base class                                   //
//                                                                   //                    //
//                                                                   //
//    Autheor : Christopher Conner (cmc@uic.edu)                     //
//    Created : 04/27/97                                             //
//    Version : 1.0                                                  //
//    Changed :                                                      //
//                                                                   //
///////////////////////////////////////////////////////////////////////

// Root Classes



// Phobos Classes

#include "TPhWA98Fam.h" 

class TPhWA98Pad : public TPhPad
{

public:
    TPhWA98Pad() {fFamily = TPhWA98Family::kFamID; }
    
    virtual ~TPhWA98Pad() {}

	Int_t GetPreampChip()     {return fPreampChip;     }
	Int_t GetChipChannel()    {return fChipChannel;    }
	Int_t GetPadPhi()         {return fPadPhi;         } 
	Int_t GetPadRadius()      {return fPadRadius;      }
	Double_t GetCenterRadius()   {return fCenterRadius;   }
	Double_t GetCenterPhi()      {return fCenterPhi;      }
	Double_t GetPseudorapidity() {return fPseudorapidity; }

	void    SetPreampChip     (Int_t x)  {fPreampChip     = x; }
	void    SetChipChannel    (Int_t x)  {fChipChannel    = x; }
	void    SetPadPhi         (Int_t x)  {fPadPhi         = x; } 
	void    SetPadRadius      (Int_t x)  {fPadRadius      = x; }
	void    SetCenterRadius   (Double_t x)  {fCenterRadius   = x; }
	void    SetCenterPhi      (Double_t x)  {fCenterPhi      = x; }
	void    SetPseudorapidity (Double_t x)  {fPseudorapidity = x; }

    void    Draw(Int_t color=-1);
    void    DrawPoint(Int_t color=-1);
protected:
	Int_t fPreampChip;     // Preamp Chip number
	Int_t fChipChannel;    // Channel number on Chip
	Int_t fPadPhi;         // Pad Phi position
	Int_t fPadRadius;      // Pad Radius position
	Double_t fCenterRadius;   // Radius of Center of Pad 
	Double_t fCenterPhi;      // Phi of Center of Pad
	Double_t fPseudorapidity; // Pseudorapidity = -log(tan(theta/2) 
	                         // assuming z = 328.5 mm
    

    ClassDef(TPhWA98Pad,1) // Individual pad geometry for WA98 detector
};

#endif
